//
//  AIStatusGroup.m
//  Adium
//
//  Created by Evan Schoenberg on 11/26/05.
//

#import "AIStatusGroup.h"
#import "AIStatus.h"
#import <Adium/AIObject.h>

@interface AIStatusGroup (PRIVATE)
- (NSNumber *)uniqueStatusID;
@end

@implementation AIStatusGroup

/*!
 * @brief Initialize with coder
 */
- (id)initWithCoder:(NSCoder *)decoder
{
	if ((self = [super init])) {
		if ([decoder allowsKeyedCoding]) {
			//Can decode keys in any order
			statusDict = [[decoder decodeObjectForKey:@"AIStatusDict"] mutableCopy];
			containedStatusItems = [[decoder decodeObjectForKey:@"ContainedStatusItems"] mutableCopy];

		} else {
			// Must decode keys in same order as encodeWithCoder:		
			statusDict = [[decoder decodeObject] mutableCopy];
			containedStatusItems = [[decoder decodeObject] mutableCopy];
		}

		if (!containedStatusItems) containedStatusItems = [[NSMutableArray alloc] init];
		if (!statusDict) statusDict = [[NSMutableDictionary alloc] init];
	}
	
	return self;
}

- (void)dealloc
{
	[containedStatusItems release];
	[statusDict release];
	
	[super dealloc];
}

- (id)mutableCopy
{
	return [containedStatusItems mutableCopy];
}

/*!
 * @brief Encode with Coder
 */
- (void)encodeWithCoder:(NSCoder *)encoder
{
	encoding = YES;

	//Ensure we have a unique status ID before encoding. We set encoding = YES so it won't trigger further saving/encoding.
	[self uniqueStatusID];

	if ([encoder allowsKeyedCoding]) {
		[encoder encodeObject:statusDict forKey:@"AIStatusDict"];
        [encoder encodeObject:containedStatusItems forKey:@"ContainedStatusItems"];
		
    } else {
		[encoder encodeObject:statusDict];
        [encoder encodeObject:containedStatusItems];
    }
	
	encoding = NO;
}

- (void)forwardInvocation:(NSInvocation *)invocation
{
    SEL aSelector = [invocation selector];
    if ([containedStatusItems respondsToSelector:aSelector]) {
		//First try to act like an array
        [invocation invokeWithTarget:containedStatusItems];
	} else if ([[[[AIObject sharedAdiumInstance] statusController] defaultInitialStatusState] respondsToSelector:aSelector]) {
		//Then try to act like an AIStatus
		[invocation invokeWithTarget:[[[AIObject sharedAdiumInstance] statusController] defaultInitialStatusState]];
	} else{
		//Then give up
        [self doesNotRecognizeSelector:aSelector];
	}
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)aSelector
{
	if (![self respondsToSelector:aSelector]) {
		if ([containedStatusItems respondsToSelector:aSelector]) {
			return [containedStatusItems methodSignatureForSelector:aSelector];
		} else if ([[[[AIObject sharedAdiumInstance] statusController] defaultInitialStatusState] respondsToSelector:aSelector]) {
			return [[[[AIObject sharedAdiumInstance] statusController] defaultInitialStatusState] methodSignatureForSelector:aSelector];
		} else {
			return [super methodSignatureForSelector:aSelector];
		}

	} else {
		return [super methodSignatureForSelector:aSelector];
	}
}

#pragma mark Unique status ID

/*!
 * @brief Return the unique status ID for this status as an integer
 *
 * The unique ID will not be assigned if necessary. -1 is returned if no unique ID has been assigned previously.
 */
- (int)preexistingUniqueStatusID
{
	NSNumber	*uniqueStatusID = [statusDict objectForKey:STATUS_UNIQUE_ID];
	
	return uniqueStatusID ? [uniqueStatusID intValue] : -1;
}

- (void)setUniqueStatusID:(NSNumber *)inUniqueStatusID
{
	if (inUniqueStatusID) {
		[statusDict setObject:inUniqueStatusID
					   forKey:STATUS_UNIQUE_ID];		
	} else {
		[statusDict removeObjectForKey:STATUS_UNIQUE_ID];
	}
	
	if (!encoding) {
		[[[AIObject sharedAdiumInstance] statusController] statusStateDidSetUniqueStatusID];
	}
}

/*!
 * @brief Return a unique ID for this status
 *
 * The unique ID will be assigned if necessary.
 */
- (NSNumber *)uniqueStatusID
{
	NSNumber	*uniqueStatusID = [statusDict objectForKey:STATUS_UNIQUE_ID];
	if (!uniqueStatusID) {
		uniqueStatusID = [[[AIObject sharedAdiumInstance] statusController] nextUniqueStatusID];
		[self setUniqueStatusID:uniqueStatusID];
	}
	
	return uniqueStatusID;
}


@end
