/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// $Id: AIAccountController.m 13190 2005-08-22 01:10:57Z evands $

#import "AIAccountController.h"
#import "AIContactController.h"
#import "AIContentController.h"
#import "AILoginController.h"
#import "AIPreferenceController.h"
#import "AIStatusController.h"
#import "ESAccountPasswordPromptController.h"
#import "ESProxyPasswordPromptController.h"
#import <AIUtilities/AIDictionaryAdditions.h>
#import <AIUtilities/AIKeychain.h>
#import <AIUtilities/AIKeychainOld.h>
#import <AIUtilities/AIMenuAdditions.h>
#import <AIUtilities/CBObjectAdditions.h>
#import <AIUtilities/ESImageAdditions.h>
#import <Adium/AIAccount.h>
#import <Adium/AIChat.h>
#import <Adium/AIContentMessage.h>
#import <Adium/AIContentObject.h>
#import <Adium/AIListContact.h>
#import <Adium/AIListObject.h>
#import <Adium/AIMetaContact.h>
#import <Adium/AIService.h>
#import <Adium/AIServiceIcons.h>
#import <Adium/AIStatusIcons.h>

//Paths and Filenames
#define PREF_GROUP_PREFERRED_ACCOUNTS   @"Preferred Accounts"
#define ACCOUNT_DEFAULT_PREFS			@"AccountPrefs"

//Preference keys
#define TOP_ACCOUNT_ID					@"TopAccountID"   	//Highest account object ID
#define ACCOUNT_LIST					@"Accounts"   		//Array of accounts
#define ACCOUNT_TYPE					@"Type"				//Account type
#define ACCOUNT_SERVICE					@"Service"			//Account service
#define ACCOUNT_UID						@"UID"				//Account UID
#define ACCOUNT_OBJECT_ID				@"ObjectID"   		//Account object ID

//Other
#define KEY_PREFERRED_SOURCE_ACCOUNT	@"Preferred Account"

//In AdiumFramework in 0.9
#define NEW_ACCOUNT_DISPLAY_TEXT			AILocalizedStringFromTable(@"<New Account>", @"AdiumFramework", "Placeholder displayed as the name of a new account")

//XXX
#import "AIAccountListWindowController.h"

@interface AIAccountController (PRIVATE)
- (void)loadAccounts;
- (void)_addMenuItemsToMenu:(NSMenu *)menu withTarget:(id)target forAccounts:(NSArray *)accounts;
- (NSArray *)_accountsForSendingContentType:(NSString *)inType toListObject:(AIListObject *)inObject preferred:(BOOL)inPreferred includeOffline:(BOOL)includeOffline;

- (void)_addAccountMenuItemsForPlugin:(id<AccountMenuPlugin>)accountMenuPlugin;
- (void)_removeAccountMenuItemsForPlugin:(id<AccountMenuPlugin>)accountMenuPlugin;
- (void)_updateMenuItem:(NSMenuItem *)menuItem forAccount:(AIAccount *)account forPlugin:(id<AccountMenuPlugin>)accountMenuPlugin;
- (NSMenuItem *)_menuItemForAccount:(AIAccount *)account fromArray:(NSArray *)accountMenuItemArray;
- (void)rebuildAllAccountMenuItems;

- (BOOL)_account:(AIAccount *)account canSendContentType:(NSString *)inType toListObject:(AIListObject *)inObject preferred:(BOOL)inPreferred includeOffline:(BOOL)includeOffline;

- (void)_upgradePasswords;

- (void)_prepareAccountMenus;
- (NSMenu *)actionsMenuForAccount:(AIAccount *)inAccount;

@end

@implementation AIAccountController

//init
- (void)initController
{
    availableServiceDict = [[NSMutableDictionary alloc] init];
	availableServiceTypeDict = [[NSMutableDictionary alloc] init];
    accountArray = nil;
    lastAccountIDToSendContent = [[NSMutableDictionary alloc] init];
	accountMenuItemArraysDict = [[NSMutableDictionary alloc] init];
	accountMenuPluginsDict = [[NSMutableDictionary alloc] init];
	_cachedActiveServices = nil;

	[[adium notificationCenter] addObserver:self
								   selector:@selector(rebuildAllAccountMenuItems)
									   name:AIStatusIconSetDidChangeNotification
									 object:nil];
	[[adium notificationCenter] addObserver:self
								   selector:@selector(rebuildAllAccountMenuItems)
									   name:AIServiceIconSetDidChangeNotification
									 object:nil];
	
	
	//Default account preferences
	[[adium preferenceController] registerDefaults:[NSDictionary dictionaryNamed:ACCOUNT_DEFAULT_PREFS forClass:[self class]]
										  forGroup:PREF_GROUP_ACCOUNTS];
}

//Finish initialization once other controllers have set themselves up
- (void)finishIniting
{   
	//Observe account changes, both list and online/connecting/offline
    [[adium notificationCenter] addObserver:self
								   selector:@selector(accountListChanged:)
									   name:Account_ListChanged
									 object:nil];
    [[adium contactController] registerListObjectObserver:self];
	
    //Load the user accounts
    [self loadAccounts];
    
    //Observe content (for accountForSendingContentToContact)
    [[adium notificationCenter] addObserver:self
                                   selector:@selector(didSendContent:)
                                       name:CONTENT_MESSAGE_SENT
                                     object:nil];

	/* Temporary upgrade code! */
	NSUserDefaults	*userDefaults = [NSUserDefaults standardUserDefaults];
	NSNumber	*didPasswordUpgrade = [userDefaults objectForKey:@"Adium:Did Password Upgrade"];
	if(!didPasswordUpgrade || ![didPasswordUpgrade boolValue]){
		[userDefaults setObject:[NSNumber numberWithBool:YES]
						 forKey:@"Adium:Did Password Upgrade"];
		[userDefaults synchronize];

		if([accountArray count]){
			[self _upgradePasswords];
		}
	}

	[self _prepareAccountMenus];
}

//close
- (void)closeController
{
    //Disconnect all accounts
    [self disconnectAllAccounts];
    
    //Remove observers (otherwise, every account added will be a duplicate next time around)
    [[adium notificationCenter] removeObserver:self];
	[[adium contactController] unregisterListObjectObserver:self];
}

- (void)dealloc
{
	//Cleanup
    [accountArray release];
	[unloadableAccounts release];
    [availableServiceDict release];
    [lastAccountIDToSendContent release];
	[accountMenuItemArraysDict release];
	[accountMenuPluginsDict release];

	[_cachedActiveServices release]; _cachedActiveServices = nil;
	
	[super dealloc];
}

//Account Storage ------------------------------------------------------------------------------------------------------
#pragma mark Account Storage
//Loads the saved accounts
- (void)loadAccounts
{
    NSArray			*accountList;
	NSEnumerator	*enumerator;
	NSDictionary	*accountDict;

	//Close down any existing accounts
	[accountArray release]; accountArray = [[NSMutableArray alloc] init];
	[unloadableAccounts release]; unloadableAccounts = [[NSMutableArray alloc] init];	
	
	accountList = [[adium preferenceController] preferenceForKey:ACCOUNT_LIST group:PREF_GROUP_ACCOUNTS];
	
    //Create an instance of every saved account
	enumerator = [accountList objectEnumerator];
	while(accountDict = [enumerator nextObject]){
		NSString		*serviceID = [accountDict objectForKey:ACCOUNT_TYPE];
        AIAccount		*newAccount;
        AIService		*service;
		NSString		*accountUID;
		NSString		*internalObjectID;
		
		//TEMPORARY UPGRADE CODE  0.63 -> 0.70 (Account format changed)
		//####################################
		if([serviceID isEqualToString:@"AIM-LIBGAIM"]){
			NSString 	*uid = [accountDict objectForKey:ACCOUNT_UID];
			if(uid && [uid length]){
				const char	firstCharacter = [uid characterAtIndex:0];
				
				if([uid hasSuffix:@"@mac.com"]){
					serviceID = @"libgaim-oscar-Mac";
				}else if(firstCharacter >= '0' && firstCharacter <= '9'){
					serviceID = @"libgaim-oscar-ICQ";
				}else{
					serviceID = @"libgaim-oscar-AIM";
				}
			}
		}else if([serviceID isEqualToString:@"GaduGadu-LIBGAIM"]){
			serviceID = @"libgaim-Gadu-Gadu";
		}else if([serviceID isEqualToString:@"Jabber-LIBGAIM"]){
			serviceID = @"libgaim-Jabber";
		}else if([serviceID isEqualToString:@"MSN-LIBGAIM"]){
			serviceID = @"libgaim-MSN";
		}else if([serviceID isEqualToString:@"Napster-LIBGAIM"]){
			serviceID = @"libgaim-Napster";
		}else if([serviceID isEqualToString:@"Novell-LIBGAIM"]){
			serviceID = @"libgaim-GroupWise";
		}else if([serviceID isEqualToString:@"Sametime-LIBGAIM"]){
			serviceID = @"libgaim-Sametime";
		}else if([serviceID isEqualToString:@"Yahoo-LIBGAIM"]){
			serviceID = @"libgaim-Yahoo!";
		}else if([serviceID isEqualToString:@"Yahoo-Japan-LIBGAIM"]){
			serviceID = @"libgaim-Yahoo!-Japan";
		}
		//####################################

		//XXX: Temporary Rendezvous -> Bonjour code
		if([serviceID isEqualToString:@"rvous-libezv"]){
			serviceID = @"bonjour-libezv";
		}
		
		//Fetch the account service, UID, and ID
		service = [self serviceWithUniqueID:serviceID];
		accountUID = [accountDict objectForKey:ACCOUNT_UID];
		internalObjectID = [accountDict objectForKey:ACCOUNT_OBJECT_ID];
		
        //Create the account and add it to our array
        if(service && accountUID && [accountUID length]){
			if(newAccount = [self createAccountWithService:service UID:accountUID internalObjectID:internalObjectID]){
                [accountArray addObject:newAccount];
            }else{
				[unloadableAccounts addObject:accountDict];
			}
        }
    }
	
	//Broadcast an account list changed notification
    [[adium notificationCenter] postNotificationName:Account_ListChanged object:nil userInfo:nil];
}

//Save the accounts
- (void)saveAccounts
{
	NSMutableArray	*flatAccounts = [NSMutableArray array];
	NSEnumerator	*enumerator;
	AIAccount		*account;
	
	//Build a flattened array of the accounts
	enumerator = [accountArray objectEnumerator];
	while(account = [enumerator nextObject]){
		NSMutableDictionary		*flatAccount = [NSMutableDictionary dictionary];
		
		[flatAccount setObject:[[account service] serviceCodeUniqueID] forKey:ACCOUNT_TYPE]; 	//Unique plugin ID
		[flatAccount setObject:[[account service] serviceID] forKey:ACCOUNT_SERVICE];	    	//Shared service ID
		[flatAccount setObject:[account UID] forKey:ACCOUNT_UID];		    					//Account UID
		[flatAccount setObject:[account internalObjectID] forKey:ACCOUNT_OBJECT_ID];  			//Account Object ID
		
		[flatAccounts addObject:flatAccount];
	}
	
	//Add any unloadable accounts so they're not lost
	[flatAccounts addObjectsFromArray:unloadableAccounts];
	
	//Save
	[[adium preferenceController] setPreference:flatAccounts forKey:ACCOUNT_LIST group:PREF_GROUP_ACCOUNTS];
	
	//Broadcast an account list changed notification
	[[adium notificationCenter] postNotificationName:Account_ListChanged object:nil userInfo:nil];
}

//Returns a new account of the specified type (Unique service plugin ID)
- (AIAccount *)createAccountWithService:(AIService *)service UID:(NSString *)inUID internalObjectID:(NSString *)internalObjectID
{	
	//Filter the UID
	inUID = [service filterUID:inUID removeIgnoredCharacters:YES];
	
	//If no object ID is provided, use the next largest integer
	if(!internalObjectID){
		int	topAccountID = [[[adium preferenceController] preferenceForKey:TOP_ACCOUNT_ID group:PREF_GROUP_ACCOUNTS] intValue];
		internalObjectID = [NSString stringWithFormat:@"%i",topAccountID];
		[[adium preferenceController] setPreference:[NSNumber numberWithInt:topAccountID + 1]
											 forKey:TOP_ACCOUNT_ID
											  group:PREF_GROUP_ACCOUNTS];
	}
	
	//Create the account
	return([service accountWithUID:inUID internalObjectID:internalObjectID]);
}


//Services -------------------------------------------------------------------------------------------------------
#pragma mark Services
//Sort an array of services alphabetically by their description
int _alphabeticalServiceSort(id service1, id service2, void *context)
{
	return([(NSString *)[service1 longDescription] caseInsensitiveCompare:(NSString *)[service2 longDescription]]);
}

//Return the available services.  These are used for account creation.
- (NSArray *)availableServices
{
	return([[availableServiceDict allValues] sortedArrayUsingFunction:_alphabeticalServiceSort context:nil]);
}

//Return the active services (services for which there is an account).  These are used for contact creation and determining if
//the service of accounts and contacts should be presented to the user.
//Simultaneously determine if any active service can 
//If includeCompatible is YES, services not being used but in the same service class as one that is will be included
- (NSArray *)activeServicesIncludingCompatibleServices:(BOOL)includeCompatible
{
	if(!_cachedActiveServices){
		NSMutableArray	*serviceArray = [NSMutableArray array];
		NSEnumerator	*enumerator = [accountArray objectEnumerator];
		AIAccount		*account;

		//Build an array of all currently used services
		while(account = [enumerator nextObject]){
			if(includeCompatible){
				NSEnumerator	*serviceEnumerator;
				AIService		*accountService;
				
				//Add all services that are of the same class as the user's account (So, all compatible services)
				serviceEnumerator = [[self servicesWithServiceClass:[[account service] serviceClass]] objectEnumerator];
				while(accountService = [serviceEnumerator nextObject]){
					//Prevent any service from going in twice
					if(![serviceArray containsObject:accountService]){
						[serviceArray addObject:accountService];
					}
					
				}
			}else{
				if(![serviceArray containsObject:[account service]]){
					[serviceArray addObject:[account service]];
				}
			}
		}
		
		//Sort
		_cachedActiveServices = [[serviceArray sortedArrayUsingFunction:_alphabeticalServiceSort context:nil] retain];
	}
	
	return(_cachedActiveServices);
}

//Returns the specified service controller
- (AIService *)serviceWithUniqueID:(NSString *)identifier
{
    return([availableServiceDict objectForKey:identifier]);
}

//Return the first service with the specified serviceID
- (AIService *)firstServiceWithServiceID:(NSString *)serviceID
{
	NSEnumerator	*enumerator = [availableServiceDict objectEnumerator];
	AIService		*service;
	
	while(service = [enumerator nextObject]){
		if([[service serviceID] isEqualToString:serviceID]) break;
	}
	
	return(service);
}

- (NSArray *)servicesWithServiceClass:(NSString *)serviceClass
{
	NSEnumerator	*enumerator = [availableServiceDict objectEnumerator];
	AIService		*service;
	NSMutableArray	*servicesArray = [NSMutableArray array];
		
	while(service = [enumerator nextObject]){
		if([[service serviceClass] isEqualToString:serviceClass]) [servicesArray addObject:service];
	}
	
	return(servicesArray);
}

- (BOOL)serviceWithUniqueIDIsOnline:(NSString *)identifier
{
	AIService		*service = [self serviceWithUniqueID:identifier];
    NSEnumerator	*enumerator = [accountArray objectEnumerator];
    AIAccount		*account;
    
    while((account = [enumerator nextObject])){
		if(([account service] == service) &&
		   [account online]) return YES;
    }
    
    return(NO);
}

//Register service code
- (void)registerService:(AIService *)inService
{
    [availableServiceDict setObject:inService forKey:[inService serviceCodeUniqueID]];
	
	[availableServiceTypeDict setObject:inService forKey:[inService serviceID]];
}

//Returns a menu of all services.
//- Selector called on service selection is selectAccount:
//- The menu item's represented objects are the service controllers they represent
//- Format allows the description to be placed within a format string. If it is nil, the description alone will be used.
- (NSMenu *)menuOfServicesWithTarget:(id)target activeServicesOnly:(BOOL)activeServicesOnly longDescription:(BOOL)longDescription format:(NSString *)format
{
	AIServiceImportance	importance;
	unsigned			numberOfItems = 0;
	NSArray				*serviceArray;
	
	//Prepare our menu
	NSMenu *menu = [[NSMenu alloc] init];

	serviceArray = (activeServicesOnly ? [self activeServicesIncludingCompatibleServices:YES] : [self availableServices]);
	
	//Divide our menu into sections.  This helps separate less important services from the others (sorry guys!)
	for(importance = AIServicePrimary; importance <= AIServiceUnsupported; importance++){
		NSEnumerator	*enumerator;
		AIService		*service;
		unsigned		currentNumberOfItems;
		BOOL			addedDivider = NO;
		
		//Divider
		currentNumberOfItems = [menu numberOfItems];
		if (currentNumberOfItems > numberOfItems){
			[menu addItem:[NSMenuItem separatorItem]];
			numberOfItems = currentNumberOfItems + 1;
			addedDivider = YES;
		}

		//Insert a menu item for each service of this importance
		enumerator = [serviceArray objectEnumerator];
		while((service = [enumerator nextObject])){
			if([service serviceImportance] == importance){
				NSString	*description = (longDescription ?
											[service longDescription] :
											[service shortDescription]);
				
				NSMenuItem	*menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:(format ? 
																									 [NSString stringWithFormat:format,description] :
																									 description)
																							 target:target 
																							 action:@selector(selectServiceType:) 
																					  keyEquivalent:@""];
				[menuItem setRepresentedObject:service];
				[menuItem setImage:[AIServiceIcons serviceIconForService:service
																	type:AIServiceIconSmall
															   direction:AIIconNormal]];
				[menu addItem:menuItem];
				[menuItem release];
			}
		}
		
		//If we added a divider but didn't add any items, remove it
		currentNumberOfItems = [menu numberOfItems];
		if (addedDivider && (currentNumberOfItems <= numberOfItems) && (currentNumberOfItems > 0)){
			[menu removeItemAtIndex:(currentNumberOfItems-1)];
		}
	}
	
	return([menu autorelease]);
}	

//Accounts -------------------------------------------------------------------------------------------------------
#pragma mark Accounts
//Returns all available accounts
- (NSArray *)accountArray
{
    return(accountArray);
}

//Searches the account list for the specified account
- (AIAccount *)accountWithInternalObjectID:(NSString *)objectID
{
    NSEnumerator	*enumerator = [accountArray objectEnumerator];
    AIAccount		*account = nil;
	
	//XXX - Temporary Upgrade code for account internalObjectIDs stored as NSNumbers 0.7x -> 0.8 -ai
	if(![objectID isKindOfClass:[NSString class]]){
		if([objectID isKindOfClass:[NSNumber class]]){
			objectID = [NSString stringWithFormat:@"%i",[(NSNumber *)objectID intValue]];
		}else{
			objectID = nil; //Unrecognizable, ignore
		}
	}
    
    while(objectID && (account = [enumerator nextObject])){
        if([objectID isEqualToString:[account internalObjectID]]) break;
    }
    
    return(account);
}

//Searches the account list for accounts with the specified service ID
- (NSArray *)accountsWithService:(AIService *)service
{
	NSMutableArray	*array = [NSMutableArray array];
    NSEnumerator	*enumerator = [accountArray objectEnumerator];
    AIAccount		*account;
    
    while((account = [enumerator nextObject])){
		if([account service] == service) [array addObject:account];
    }
    
    return(array);
}

//Searches the account list for accounts with the specified service's serviceClass
//This could use other methods but that would require alloc'ing significantly more NSArrays, so we consolidate
//for efficiency.
- (NSArray *)accountsWithServiceClassOfService:(AIService *)service
{
	return([self accountsWithServiceClass:[service serviceClass]]);
}

- (NSArray *)accountsWithServiceClass:(NSString *)serviceClass
{
	NSMutableArray	*accountsArray = [NSMutableArray array];
	NSEnumerator	*enumerator = [availableServiceDict objectEnumerator];
    AIService		*aService;
	
	//Enumerate all available services
	while(aService = [enumerator nextObject]){
		//Find matching serviceClasses
		if([[aService serviceClass] isEqualToString:serviceClass]){
			
			//Find matching accounts
			NSEnumerator	*enumerator = [accountArray objectEnumerator];
			AIAccount		*account;
			
			while((account = [enumerator nextObject])){
				if ([account service] == aService){
					[accountsArray addObject:account];
				}
			}
		}
	}
	
	return(accountsArray);
}

- (AIAccount *)firstAccountWithService:(AIService *)service
{
    NSEnumerator	*enumerator = [accountArray objectEnumerator];
    AIAccount		*account;
    
    while((account = [enumerator nextObject])){
		if([account service] == service) break;
    }
    
    return(account);
}

//Returns a new default account
- (AIAccount *)defaultAccount
{
	return([self createAccountWithService:[self serviceWithUniqueID:@"libgaim-oscar-AIM"] UID:@"" internalObjectID:nil]);
}

- (BOOL)anOnlineAccountCanCreateGroupChats
{
	NSEnumerator	*enumerator;
	AIAccount		*account;
	BOOL			anOnlineAccountCanCreateGroupChats;
	
	anOnlineAccountCanCreateGroupChats = NO;
	
    enumerator = [accountArray objectEnumerator];
    while(account = [enumerator nextObject]){	
		if ([account online] && [[account service] canCreateGroupChats]){
			anOnlineAccountCanCreateGroupChats = YES;
			break;
		}
	}
	
	return(anOnlineAccountCanCreateGroupChats);
}

//Account Editing ------------------------------------------------------------------------------------------------------
#pragma mark Account Editing
//Create a new default account
- (AIAccount *)newAccountAtIndex:(int)index
{
	if(index == -1) index = [accountArray count];
	
    NSParameterAssert(accountArray != nil);
    NSParameterAssert(index >= 0 && index <= [accountArray count]);
    
    AIAccount	*newAccount = [self defaultAccount];
    
	[self insertAccount:newAccount atIndex:index save:YES];
		
    return(newAccount);
}

- (AIAccount *)newAccountAtIndex:(int)index forService:(AIService *)service
{
	if(index == -1) index = [accountArray count];

    NSParameterAssert(accountArray != nil);
    NSParameterAssert(index >= 0 && index <= [accountArray count]);

	AIAccount *newAccount;
	
	if(service){
		newAccount = [self createAccountWithService:service	UID:@"" internalObjectID:nil];
	}else{
		newAccount = [self defaultAccount];
	}
	
	[self insertAccount:newAccount atIndex:index save:YES];
	
    return(newAccount);
}

//Insert an account
- (void)insertAccount:(AIAccount *)inAccount atIndex:(int)index save:(BOOL)shouldSave
{    
	if(index == -1) index = [accountArray count];

    NSParameterAssert(inAccount != nil);
    NSParameterAssert(accountArray != nil);
    NSParameterAssert(index >= 0 && index <= [accountArray count]);
    
    //Insert the account
	if ([accountArray count]){
		[accountArray insertObject:inAccount atIndex:index];
	}else{
		[accountArray addObject:inAccount];
	}
	
	if (shouldSave){
		[self saveAccounts];
	}
}

//Delete an account
- (void)deleteAccount:(AIAccount *)inAccount save:(BOOL)shouldSave
{
    NSParameterAssert(inAccount != nil);
    NSParameterAssert(accountArray != nil);

	//Don't let the account dealloc until we have a chance to notify everyone that it's gone
	[inAccount retain]; 

	//Let the account take any action it wants before being deleted, such as disconnecting
	[inAccount willBeDeleted];
	
	//Remove from our array
	[accountArray removeObject:inAccount];
	
	//Clean up the keychain -- forget the stored password
	[self forgetPasswordForAccount:inAccount];
	
	//Save if appropriate
	if (shouldSave){
		[self saveAccounts];
	}
	
	//Cleanup
	[inAccount release];
}

//Re-order an account on the list
- (int)moveAccount:(AIAccount *)account toIndex:(int)destIndex
{
    int sourceIndex = [accountArray indexOfObject:account];
    
    //Remove the account
    [account retain];
    [accountArray removeObject:account];
    
    //Re-insert the account
    if(destIndex > sourceIndex){
        destIndex -= 1;
    }
    [accountArray insertObject:account atIndex:destIndex];
    [account release];
    
    [self saveAccounts];
    
    return(destIndex);
}

//The account list changed.
- (void)accountListChanged:(NSNotification *)notification
{
	//Clear our cached active service types
	[_cachedActiveServices release]; _cachedActiveServices = nil;

	// Perform a full rebuild rather than trying to figure out what is different.
	[self rebuildAllAccountMenuItems];
}

//Preferred Source Accounts --------------------------------------------------------------------------------------------
#pragma mark Preferred Source Accounts
//Returns the preferred choice for sending content to the passed list object
//When presenting the user with a list of accounts, this should be the one selected by default
- (AIAccount *)preferredAccountForSendingContentType:(NSString *)inType toContact:(AIListContact *)inContact 
{
	return ([self preferredAccountForSendingContentType:inType toContact:inContact includeOffline:NO]);
}

- (AIAccount *)preferredAccountForSendingContentType:(NSString *)inType toContact:(AIListContact *)inContact includeOffline:(BOOL)includeOffline
{
	AIAccount		*account;
	
	//If passed a contact, we have a few better ways to determine the account than just using the first
    if(inContact){
		//If we've messaged this object previously, and the account we used to message it is online, return that account
        NSString *accountID = [inContact preferenceForKey:KEY_PREFERRED_SOURCE_ACCOUNT
													group:PREF_GROUP_PREFERRED_ACCOUNTS];
        if(accountID && (account = [self accountWithInternalObjectID:accountID])){
            if([account availableForSendingContentType:inType toContact:inContact]){
                return(account);
            }
        }
		
		//If inObject is an AIListContact return the account the object is on
		if(account = [inContact account]){
			if([account availableForSendingContentType:inType toContact:inContact]){
				return(account);
			}
		}
		
		//Return the last account used to message someone on this service
		NSString	*lastAccountID = [lastAccountIDToSendContent objectForKey:[[inContact service] serviceID]];
		if(lastAccountID && (account = [self accountWithInternalObjectID:lastAccountID])){
			if([account availableForSendingContentType:inType toContact:nil] || includeOffline){
				return(account);
			}
		}
		
		if (includeOffline){
			//If inObject is an AIListContact return the account the object is on even if the account is offline
			if(account = [inContact account]){
				return(account);
			}
		}
	}
	
	//If the previous attempts failed, or we weren't passed a contact, use the first appropraite account
	return([self firstAccountAvailableForSendingContentType:inType
												  toContact:inContact
											 includeOffline:includeOffline]);
}

- (AIAccount *)firstAccountAvailableForSendingContentType:(NSString *)inType toContact:(AIListContact *)inContact includeOffline:(BOOL)includeOffline
{
	AIAccount		*account;
	NSEnumerator	*enumerator;
	
    if(inContact){
		//First available account in our list of the correct service type
		enumerator = [accountArray objectEnumerator];
		while(account = [enumerator nextObject]){
			if([inContact service] == [account service] &&
			   ([account availableForSendingContentType:inType toContact:nil] || includeOffline)){
				return(account);
			}
		}
		
		//First available account in our list of a compatible service type
		enumerator = [accountArray objectEnumerator];
		while(account = [enumerator nextObject]){
			if([[inContact serviceClass] isEqualToString:[account serviceClass]] &&
			   ([account availableForSendingContentType:inType toContact:nil] || includeOffline)){
				return(account);
			}
		}
	}else{
		//First available account in our list
		enumerator = [accountArray objectEnumerator];
		while(account = [enumerator nextObject]){
			if([account availableForSendingContentType:inType toContact:nil] || includeOffline){
				return(account);
			}
		}
	}
	
	
	//Can't find anything
	return(nil);
}

//Returns a menu of all accounts returned by menuItemsForAccountsWithTarget
- (NSMenu *)menuOfAccountsWithTarget:(id)target includeOffline:(BOOL)includeOffline
{
	return ([self menuOfAccountsWithTarget:target includeOffline:includeOffline onlyIfCreatingGroupChatIsSupported:NO]);
}

//Returns a menu of all accounts (optionally, which can create a group chat) returned by menuItemsForAccountsWithTarget
- (NSMenu *)menuOfAccountsWithTarget:(id)target includeOffline:(BOOL)includeOffline onlyIfCreatingGroupChatIsSupported:(BOOL)groupChatCreator
{
	NSMenu			*menu = [[NSMenu alloc] init];
	NSEnumerator	*enumerator;
	NSMenuItem		*menuItem;
	
	enumerator = [[self menuItemsForAccountsWithTarget:target includeOffline:includeOffline] objectEnumerator];
	while(menuItem = [enumerator nextObject]){
		if (!groupChatCreator || [[[menuItem representedObject] service] canCreateGroupChats]){
			[menu addItem:menuItem];
		}
	}

	if(!target) [menu setAutoenablesItems:NO];

	return([menu autorelease]);
}

//Returns an array containing menu items for all accounts. 
//- Accounts not available for sending content are disabled.
//- Selector called on account selection is selectAccount:
//- The menu item's represented objects are the AIAccounts they represent
- (NSArray *)menuItemsForAccountsWithTarget:(id)target includeOffline:(BOOL)includeOffline
{
	NSMutableArray  *menuItems = [[NSMutableArray alloc] init];
	NSEnumerator	*enumerator;
	AIAccount		*account;
	
	//We don't show service types unless the user is using multiple services
	BOOL 	multipleServices = ([[self activeServicesIncludingCompatibleServices:NO] count] > 1);
	
    //Insert a menu item for each available account
    enumerator = [accountArray objectEnumerator];
    while(account = [enumerator nextObject]){
		BOOL available = [[adium contentController] availableForSendingContentType:CONTENT_MESSAGE_TYPE
																		 toContact:nil 
																		 onAccount:account];
		
		if(available || includeOffline){
			NSMenuItem	*menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:(multipleServices ?
																								  [NSString stringWithFormat:@"%@ (%@)", [account formattedUID], [[account service] shortDescription]] :
																								  [account formattedUID])
																						  target:target
																						  action:@selector(selectAccount:)
																				   keyEquivalent:@""];
			[menuItem setRepresentedObject:account];
			[menuItem setImage:[AIServiceIcons serviceIconForObject:account
															   type:AIServiceIconSmall
														  direction:AIIconNormal]];
			[menuItem setEnabled:available];
			
			[menuItems addObject:menuItem];
			[menuItem release];
		}
    }
	
	return([menuItems autorelease]);
}

//Returns a menu of all accounts available for sending content to a list object
//- Preferred choices are placed at the top of the menu.
//- Selector called on account selection is selectAccount:
//- The menu item's represented objects are the AIAccounts they represent
- (NSMenu *)menuOfAccountsForSendingContentType:(NSString *)inType
								   toListObject:(AIListObject *)inObject
									 withTarget:(id)target
								 includeOffline:(BOOL)includeOffline
{
	NSMenu		*menu;
	NSArray		*topAccounts, *bottomAccounts;
		
	//Get the list of accounts for each section of our menu
	topAccounts = [self _accountsForSendingContentType:CONTENT_MESSAGE_TYPE
										  toListObject:inObject
											 preferred:YES
										includeOffline:includeOffline];
	bottomAccounts = [self _accountsForSendingContentType:CONTENT_MESSAGE_TYPE
											 toListObject:inObject
												preferred:NO
										   includeOffline:includeOffline];

	//Build the menu
	menu = [[NSMenu alloc] init];
	[menu setAutoenablesItems:NO];
	if([topAccounts count]) [self _addMenuItemsToMenu:menu withTarget:target forAccounts:topAccounts];
	if([topAccounts count] &&  [bottomAccounts count]) [menu addItem:[NSMenuItem separatorItem]];
	if([bottomAccounts count]) [self _addMenuItemsToMenu:menu withTarget:target forAccounts:bottomAccounts];
	
	return([menu autorelease]);
}

- (void)_addMenuItemsToMenu:(NSMenu *)menu withTarget:(id)target forAccounts:(NSArray *)accounts
{
	NSEnumerator	*enumerator = [accounts objectEnumerator];
	AIAccount		*account;
	
	while(account = [enumerator nextObject]){
		NSMenuItem	*menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:[account formattedUID]
																					 target:target
																					 action:@selector(selectAccount:)
																			  keyEquivalent:@""];
		[menuItem setRepresentedObject:account];
		[menuItem setImage:[AIServiceIcons serviceIconForObject:account
														   type:AIServiceIconSmall
													  direction:AIIconNormal]];
		[menu addItem:menuItem];
		[menuItem release];
	}
}

- (NSArray *)_accountsForSendingContentType:(NSString *)inType toListObject:(AIListObject *)inObject preferred:(BOOL)inPreferred includeOffline:(BOOL)includeOffline
{
	NSMutableArray	*sourceAccounts = [NSMutableArray array];
	NSEnumerator	*enumerator = [[[adium accountController] accountArray] objectEnumerator];
	AIAccount		*account;
	
	while(account = [enumerator nextObject]){
		if((!inObject && !inPreferred) || 
		   ([self _account:account canSendContentType:inType toListObject:inObject preferred:inPreferred includeOffline:includeOffline])){
			
			[sourceAccounts addObject:account];
		}
	}
	
	return(sourceAccounts);
}

- (BOOL)_account:(AIAccount *)account canSendContentType:(NSString *)inType toListObject:(AIListObject *)inObject preferred:(BOOL)inPreferred includeOffline:(BOOL)includeOffline
{
	BOOL			canSend = NO;
	
	if ([inObject isKindOfClass:[AIMetaContact class]]){		
		NSEnumerator	*enumerator = [[(AIMetaContact *)inObject listContacts] objectEnumerator];
		AIListObject	*containedObject;
		
		//canSend is YES if any of the contained contacts of the meta contact return YES
		while (containedObject = [enumerator nextObject]){
			if ([self _account:account canSendContentType:inType toListObject:containedObject preferred:inPreferred includeOffline:includeOffline]){
				
				canSend = YES;
				break;
			}
		}

	}else{
		if ([[inObject serviceClass] isEqualToString:[account serviceClass]]){
			BOOL			knowsObject = NO;
			BOOL			canFindObject = NO;
			AIListContact	*contactForAccount = [[adium contactController] existingContactWithService:[inObject service]
																							   account:account
																								   UID:[inObject UID]];
			
			//Does the account know this object?
			if(contactForAccount){
				knowsObject = [account availableForSendingContentType:CONTENT_MESSAGE_TYPE
															toContact:contactForAccount];
			}
			
			//Could the account find this object?
			canFindObject = [account availableForSendingContentType:CONTENT_MESSAGE_TYPE toContact:nil];
			
			if((inPreferred && knowsObject) ||						//Online and can see the object
			   (!inPreferred && !knowsObject && canFindObject) ||	//Online and may be able to see the object
			   (!inPreferred && !knowsObject && includeOffline)){	//Offline, but may be able to see the object if online
				canSend = YES;
			}
		}
	}
	
	return(canSend);
}

//Watch outgoing content, remembering the user's choice of source account
- (void)didSendContent:(NSNotification *)notification
{
	NSDictionary	*userInfo = [notification userInfo];
    AIChat			*chat = [userInfo objectForKey:@"AIChat"];
    AIListObject	*destObject = [chat listObject];
    
    if(chat && destObject){
        AIContentObject *contentObject = [userInfo objectForKey:@"AIContentObject"];
        AIAccount		*sourceAccount = (AIAccount *)[contentObject source];
        
        [destObject setPreference:[sourceAccount internalObjectID]
                           forKey:KEY_PREFERRED_SOURCE_ACCOUNT
                            group:PREF_GROUP_PREFERRED_ACCOUNTS];
        
        [lastAccountIDToSendContent setObject:[sourceAccount internalObjectID] forKey:[[destObject service] serviceID]];
    }
}


//Connection convenience methods ---------------------------------------------------------------------------------------
#pragma mark Connection Convenience Methods
//Connects all the accounts
- (void)connectAllAccounts
{
    NSEnumerator		*enumerator;
    AIAccount			*account;
    
    enumerator = [accountArray objectEnumerator];
    while((account = [enumerator nextObject])){
        if([[account supportedPropertyKeys] containsObject:@"Online"]){
            [account setPreference:[NSNumber numberWithBool:YES] forKey:@"Online" group:GROUP_ACCOUNT_STATUS];
        }
    }
}

//Disconnects all the accounts
- (void)disconnectAllAccounts
{
    NSEnumerator		*enumerator;
    AIAccount			*account;

    enumerator = [accountArray objectEnumerator];
    while((account = [enumerator nextObject])){
        if([[account supportedPropertyKeys] containsObject:@"Online"] &&
		   [[account preferenceForKey:@"Online" group:GROUP_ACCOUNT_STATUS] boolValue]){
            [account setPreference:nil forKey:@"Online" group:GROUP_ACCOUNT_STATUS];
        }
    }
}

- (BOOL)oneOrMoreConnectedAccounts
{
	NSEnumerator		*enumerator;
    AIAccount			*account;

    enumerator = [accountArray objectEnumerator];
    while((account = [enumerator nextObject])){
        if([account online]){
			return YES;
        }
    }	
	
	return NO;
}

- (BOOL)oneOrMoreConnectedOrConnectingAccounts
{
	NSEnumerator		*enumerator;
    AIAccount			*account;
	
    enumerator = [accountArray objectEnumerator];
    while((account = [enumerator nextObject])){
        if([account online] || [account integerStatusObjectForKey:@"Connecting"]){
			return YES;
        }
    }	

	return NO;	
}

//Password Storage -----------------------------------------------------------------------------------------------------
#pragma mark Password Storage

- (NSString *)_accountNameForAccount:(AIAccount *)inAccount{
	return([NSString stringWithFormat:@"%@.%@",[[inAccount service] serviceID],[inAccount internalObjectID]]);
}
- (NSString *)_passKeyForAccount:(AIAccount *)inAccount{
	if([[[adium loginController] userArray] count] > 1){
		return([NSString stringWithFormat:@"Adium.%@.%@",[[adium loginController] currentUser],[self _accountNameForAccount:inAccount]]);
	}else{
		return([NSString stringWithFormat:@"Adium.%@",[self _accountNameForAccount:inAccount]]);
	}
}
- (NSString *)_accountNameForProxyServer:(NSString *)proxyServer userName:(NSString *)userName{
	return([NSString stringWithFormat:@"%@.%@",proxyServer,userName]);
}
- (NSString *)_passKeyForProxyServer:(NSString *)proxyServer{
	if([[[adium loginController] userArray] count] > 1){
		return([NSString stringWithFormat:@"Adium.%@.%@",[[adium loginController] currentUser],proxyServer]);
	}else{
		return([NSString stringWithFormat:@"Adium.%@",proxyServer]);	
	}
}


//Save an account password
- (void)setPassword:(NSString *)inPassword forAccount:(AIAccount *)inAccount
{
    if(inPassword){
        [AIKeychain putPasswordInKeychainForService:[self _passKeyForAccount:inAccount]
                                            account:[self _accountNameForAccount:inAccount]
										   password:inPassword];
    }
}

//Fetches a saved account password (returns nil if no password is saved)
- (NSString *)passwordForAccount:(AIAccount *)inAccount
{
    NSString	*password = [AIKeychain getPasswordFromKeychainForService:[self _passKeyForAccount:inAccount]
                                                                  account:[self _accountNameForAccount:inAccount]];
    return(password);
}

//Fetches a saved account password (Prompts the user to enter if no password is saved)
- (void)passwordForAccount:(AIAccount *)inAccount notifyingTarget:(id)inTarget selector:(SEL)inSelector context:(id)inContext
{
    NSString	*password;
    
    //check the keychain for this password
    password = [AIKeychain getPasswordFromKeychainForService:[self _passKeyForAccount:inAccount]
                                                     account:[self _accountNameForAccount:inAccount]];
    
    if(password && [password length] != 0){
        //Invoke the target right away
        [inTarget performSelector:inSelector withObject:password withObject:inContext afterDelay:0.0001];
    }else{
        //Prompt the user for their password
        [ESAccountPasswordPromptController showPasswordPromptForAccount:inAccount
														notifyingTarget:inTarget
															   selector:inSelector
																context:inContext];
    }
}

//Forget a saved password
- (void)forgetPasswordForAccount:(AIAccount *)inAccount
{
    [AIKeychain removePasswordFromKeychainForService:[self _passKeyForAccount:inAccount]
											 account:[self _accountNameForAccount:inAccount]];
}

- (NSString *)passwordForProxyServer:(NSString *)server userName:(NSString *)userName
{
	NSString *password = [AIKeychain getPasswordFromKeychainForService:[self _passKeyForProxyServer:server]
															   account:[self _accountNameForProxyServer:server 
																							   userName:userName]];
	return password;
}

- (void)passwordForProxyServer:(NSString *)server userName:(NSString *)userName notifyingTarget:(id)inTarget selector:(SEL)inSelector context:(id)inContext
{
	NSString	*password;
    
    //check the keychain for this password
    password = [AIKeychain getPasswordFromKeychainForService:[self _passKeyForProxyServer:server]
                                                     account:[self _accountNameForProxyServer:server userName:userName]];

    if(password && [password length] != 0){
        //Invoke the target right away
        [inTarget performSelector:inSelector withObject:password withObject:inContext afterDelay:0.0001];    
    }else{
        //Prompt the user for their password
        [ESProxyPasswordPromptController showPasswordPromptForProxyServer:server
																 userName:userName
														  notifyingTarget:inTarget
																 selector:inSelector
																  context:inContext];
    }
}
//Save a proxy server password
- (void)setPassword:(NSString *)inPassword forProxyServer:(NSString *)server userName:(NSString *)userName
{
    if(inPassword){
        [AIKeychain putPasswordInKeychainForService:[self _passKeyForProxyServer:server]
                                            account:[self _accountNameForProxyServer:server 
																			userName:userName] password:inPassword];
    }else{
		[AIKeychain removePasswordFromKeychainForService:[self _passKeyForProxyServer:server]
												 account:[self _accountNameForProxyServer:server userName:userName]];
	}
}

- (void)_upgradePasswords
{
	AIAccount		*account;
	NSEnumerator	*enumerator = [accountArray objectEnumerator];
	
	NSRunInformationalAlertPanel(@"Adium Version Upgrade", @"This version of Adium fixes a common crash related to secure storage of your instant messaging passwords.  When you press OK below, Adium will automatically update any stored passwords to the new, more stable system.\n\nThis process will only occur once and will take a moment; you may be prompted to allow access for one or more passwords.\n\nIf Adium crashes during this upgrade, simply relaunch Adium; the process will not occur again.",nil,nil,nil);

	while(account = [enumerator nextObject]){
		NSString	*passKey = [self _passKeyForAccount:account];
		NSString	*accountName = [self _accountNameForAccount:account];
		
		//Get from old
		NSString	*password = [AIKeychainOld getPasswordFromKeychainForService:passKey
																		 account:accountName];
		
		//Store in new
		if(password){
			[AIKeychain putPasswordInKeychainForService:passKey
												account:accountName
											   password:password];
		}
	}
}

// Account Connection menus -----------------------------------------------
#pragma mark Account Connection menus

#define MENU_IMAGE_FRACTION_ONLINE  		1.00
#define MENU_IMAGE_FRACTION_CONNECTING  	0.60
#define MENU_IMAGE_FRACTION_OFFLINE  		0.30

//In AdiumFramework in 0.9
#define	ACCOUNT_CONNECT_MENU_TITLE			AILocalizedStringFromTable(@"Connect: %@", @"AdiumFramework", "Connect account prefix")
#define	ACCOUNT_DISCONNECT_MENU_TITLE		AILocalizedStringFromTable(@"Disconnect: %@", @"AdiumFramework", "Disconnect account prefix")
#define	ACCOUNT_CONNECTING_MENU_TITLE		AILocalizedStringFromTable(@"Cancel: %@", @"AdiumFramework", "Cancel current account activity prefix")

#define	ACCOUNT_DISCONNECTING_MENU_TITLE	ACCOUNT_CONNECTING_MENU_TITLE

- (void)_prepareAccountMenus
{
	[[adium statusController] registerStateMenuPlugin:self];
}

- (void)registerAccountMenuPlugin:(id<AccountMenuPlugin>)accountMenuPlugin
{
	NSNumber	*identifier = [NSNumber numberWithInt:[accountMenuPlugin hash]];
	
	[accountMenuItemArraysDict setObject:[NSMutableArray array]
								  forKey:identifier];
	[accountMenuPluginsDict setObject:accountMenuPlugin
								forKey:identifier];
	
	[self _addAccountMenuItemsForPlugin:accountMenuPlugin];
	
	//Immediately add the status submenu items by asking the statusController to rebuilt our state menus.
	[[adium statusController] rebuildAllStateMenusForPlugin:self];
}
- (void)unregisterAccountMenuPlugin:(id<AccountMenuPlugin>)accountMenuPlugin
{
	NSNumber	*identifier = [NSNumber numberWithInt:[accountMenuPlugin hash]];

	[self _removeAccountMenuItemsForPlugin:accountMenuPlugin];	
	[accountMenuItemArraysDict removeObjectForKey:identifier];
	[accountMenuPluginsDict removeObjectForKey:identifier];
}

//Togle the connection of the selected account (called by the connect/disconnnect menu item)
//MUST be called by a menu item with an account as its represented object!
- (IBAction)toggleConnection:(id)sender
{
    AIAccount   *account = [sender representedObject];
    BOOL    	online = [[account statusObjectForKey:@"Online"] boolValue];
	BOOL		connecting = [[account statusObjectForKey:@"Connecting"] boolValue];
	
	// !(online || connecting) means:
	//		If neither online nor connecting, YES - initiate a connection
	//		If either currently online or currently in the process of connecting, NO - disconnect
	//
	// Setting the preference is enough to trigger the cascade which will lead to the account taking action
	[account setPreference:[NSNumber numberWithBool:!(online || connecting)] 
					forKey:@"Online"
					 group:GROUP_ACCOUNT_STATUS];
}

//Create a new menu item for each account, updating it immediately to the proper current state.
//Store these menu items in a mutableArray associated with the plugin.
//Then, inform the plugin of the existence of the menu items so it can add them to a menu.
- (void)_addAccountMenuItemsForPlugin:(id<AccountMenuPlugin>)accountMenuPlugin
{
	NSNumber		*identifier = [NSNumber numberWithInt:[accountMenuPlugin hash]];
	NSMutableArray  *menuItemArray = [accountMenuItemArraysDict objectForKey:identifier];
	
	NSEnumerator	*enumerator;
    AIAccount		*account;
    NSMenuItem		*menuItem;
	
    //Create a menuitem for each account
    enumerator = [[self accountArray] objectEnumerator];
    while((account = [enumerator nextObject])){
		
		if([[account supportedPropertyKeys] containsObject:@"Online"]){
			//Create the account's menu item (the title will be set by_updateMenuItem:forAccount:
			menuItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@""
																			target:self
																			action:@selector(toggleConnection:)
																	 keyEquivalent:@""];
			[menuItem setRepresentedObject:account];
			[menuItemArray addObject:menuItem];
			[menuItem release];
			
			[self _updateMenuItem:menuItem forAccount:account forPlugin:accountMenuPlugin];
		}
    }
	
	//Now that we are done creating the menu items, tell the plugin about them
	[accountMenuPlugin addAccountMenuItems:menuItemArray];
}

//Retrieve the menu items for a given plugin.  Tell it we want to remove them, which should trigger
//removal from whatever menu the program is using them in.  Then, remove them from our tracking array.
- (void)_removeAccountMenuItemsForPlugin:(id<AccountMenuPlugin>)accountMenuPlugin
{
	NSNumber		*identifier = [NSNumber numberWithInt:[accountMenuPlugin hash]];
	NSMutableArray  *menuItemArray = [accountMenuItemArraysDict objectForKey:identifier];

	//Inform the plugin that we are removing the items in the this array 
	[accountMenuPlugin removeAccountMenuItems:menuItemArray];
	
	//Now clear the array
	[menuItemArray removeAllObjects];
}

// Connected / Connecting / Disconnecting / Disconnected update
- (void)_updateMenuItem:(NSMenuItem *)menuItem forAccount:(AIAccount *)account forPlugin:(id<AccountMenuPlugin>)accountMenuPlugin
{
	if(menuItem){
		NSString	*accountTitle = [account formattedUID];
		NSString	*titleFormat = nil;

		//Default to <New Account> if a name is not available
		if(!accountTitle || ![accountTitle length]) accountTitle = NEW_ACCOUNT_DISPLAY_TEXT;

		//Get the status and service icons for this account so we can add them to the menu
		NSImage	*statusIcon = [AIStatusIcons statusIconForListObject:account type:AIStatusIconList direction:AIIconNormal];
		NSImage	*serviceIcon = [AIServiceIcons serviceIconForObject:account type:AIServiceIconSmall direction:AIIconNormal];

		//Show connecting and disconnecting in the title as appropriate
		if([[account statusObjectForKey:@"Connecting"] boolValue]){
			titleFormat = ACCOUNT_CONNECTING_MENU_TITLE;

		}else if([[account statusObjectForKey:@"Disconnecting"] boolValue]){
			titleFormat = ACCOUNT_DISCONNECTING_MENU_TITLE;
			
		}else if(![accountMenuPlugin showStatusSubmenu]){
			//Only show Connect or Disconnect before the name if we aren't showing the status menu
			if([account online]){
				titleFormat = ACCOUNT_DISCONNECT_MENU_TITLE;
			}else{
				titleFormat = ACCOUNT_CONNECT_MENU_TITLE;				
			}
		}

		//Composite the status icon and service icon together
		//We're only allowed one icon per menu item, so we need to combine our separate icons into a single image
		NSSize	statusSize = [statusIcon size];
		NSSize	serviceSize = [serviceIcon size];
		NSSize	compositeSize = NSMakeSize(statusSize.width + serviceSize.width + 1,
										   statusSize.height > serviceSize.height ? statusSize.height : serviceSize.height);
		NSRect	compositeRect = NSMakeRect(0, 0, compositeSize.width, compositeSize.height);

		//Render the composite image
		NSImage	*composite = [[NSImage alloc] initWithSize:compositeSize];
		[composite lockFocus];
		[statusIcon drawInRect:compositeRect atSize:[statusIcon size] position:IMAGE_POSITION_LEFT fraction:1.0];
		[serviceIcon drawInRect:compositeRect atSize:[serviceIcon size] position:IMAGE_POSITION_RIGHT fraction:1.0];
		[composite unlockFocus];

		//Update the menu item
		[[menuItem menu] setMenuChangedMessagesEnabled:NO];
		[menuItem setTitle:(titleFormat ? [NSString stringWithFormat:titleFormat,accountTitle] : accountTitle)];
		[menuItem setImage:composite];		
		[menuItem setEnabled:/*(![[account statusObjectForKey:@"Connecting"] boolValue] &&
							  ![[account statusObjectForKey:@"Disconnecting"] boolValue])*/YES];
		[[menuItem menu] setMenuChangedMessagesEnabled:YES];

		//Cleanup
		[composite release];
	}
}

//Remove all current account menu items for all account menu item plugins, then create a new, current set.
- (void)rebuildAllAccountMenuItems
{
	NSEnumerator			*enumerator = [accountMenuPluginsDict objectEnumerator];
	id<AccountMenuPlugin>   accountMenuPlugin;
	while (accountMenuPlugin = [enumerator nextObject]) {
		[self _removeAccountMenuItemsForPlugin:accountMenuPlugin];
		[self _addAccountMenuItemsForPlugin:accountMenuPlugin];
	}
	
	//Now recreate the status submenus
	[[adium statusController] rebuildAllStateMenusForPlugin:self];
}

//Account status changed, update our menu
- (NSSet *)updateListObject:(AIListObject *)inObject keys:(NSSet *)inModifiedKeys silent:(BOOL)silent
{
    if([inObject isKindOfClass:[AIAccount class]] && 
		([inModifiedKeys containsObject:@"Online"] ||
		 [inModifiedKeys containsObject:@"Connecting"] ||
		 [inModifiedKeys containsObject:@"Disconnecting"] ||
		 [inModifiedKeys containsObject:@"IdleSince"] ||
		 [inModifiedKeys containsObject:@"StatusState"])){
		
		//Enumerate all arrays of menu items (for all plugins)
		NSEnumerator	*enumerator;
		NSArray			*accountMenuItemArray;
		NSMenu			*actionsSubmenu = nil;		
		NSNumber		*identifier;
		BOOL			onlineChanged = [inModifiedKeys containsObject:@"Online"];

		if(onlineChanged){
			//If we are online, find the actions submenu if appropriate; if not, it remains nil to clear the submenu
			if([inObject online]){
				actionsSubmenu = [self actionsMenuForAccount:(AIAccount *)inObject];
			}
		}
		
		enumerator = [accountMenuItemArraysDict keyEnumerator];
		while(identifier = [enumerator nextObject]){
			id<AccountMenuPlugin>	accountMenuPlugin;
			NSMenuItem				*menuItem;

			accountMenuItemArray = [accountMenuItemArraysDict objectForKey:identifier];
			accountMenuPlugin = [accountMenuPluginsDict objectForKey:identifier];
			
			//Find the menu item for this account in this array
			menuItem = [self _menuItemForAccount:(AIAccount *)inObject
									   fromArray:accountMenuItemArray];
			//Update it
			[self _updateMenuItem:menuItem forAccount:(AIAccount *)inObject forPlugin:accountMenuPlugin];

			//Update the actions submenu if the online state changed and this is not a status-submenu-showing menuItem
			if(onlineChanged && ![accountMenuPlugin showStatusSubmenu]){
				[menuItem setSubmenu:[[actionsSubmenu copy] autorelease]];
			}
		}
    }
	
    //We don't change any keys
    return(nil);
}

//Given a target account and an array of menu items, find the menu item for that account
- (NSMenuItem *)_menuItemForAccount:(AIAccount *)account fromArray:(NSArray *)accountMenuItemArray
{
	NSEnumerator	*enumerator;
	NSMenuItem		*menuItem;
    NSMenuItem		*targetMenuItem = nil;
	
	//Find the menu
	enumerator = [accountMenuItemArray objectEnumerator];
	while((menuItem = [enumerator nextObject])){    
		if([menuItem representedObject] == account){
			targetMenuItem = menuItem;
			break;
		}
	}
	
	return targetMenuItem;	
}

- (NSMenu *)actionsMenuForAccount:(AIAccount *)inAccount
{
	NSArray	*accountActionMenuItems = [inAccount accountActionMenuItems];
	NSMenu	*actionsSubmenu = nil;

	//Only proceed if we got at least one menu item
	if(accountActionMenuItems && [accountActionMenuItems count]){
		NSEnumerator	*enumerator;
		NSMenuItem		*menuItem;

		actionsSubmenu = [[[NSMenu allocWithZone:[NSMenu zone]] init] autorelease];
		[actionsSubmenu setTitle:@"Actions"]; //Not localized, just used internally

		//Now add each item to the submenu
		enumerator = [accountActionMenuItems objectEnumerator];
		while(menuItem = [enumerator nextObject]){
			[actionsSubmenu addItem:menuItem];
		}
	}

	return actionsSubmenu;
}	

/*!
 * @brief Add the passed state menu items; also take this opportunity to add account-specific items for online accounts
 *
 * When given menu items, make a submenu out of them for each account with a represented object referring to that account.
 * We only add the status menu items in a submenu if more than one account exists, as with only one account it would be redundant.
 */
- (void)addStateMenuItems:(NSArray *)menuItemArray
{
	NSArray				*originalMenuItemArray;
	NSMutableDictionary	*temporaryMenuDict = [NSMutableDictionary dictionary];
	NSEnumerator		*enumerator;
	AIAccount			*account;
	NSArray				*accountMenuItemArray;
	int					accountArrayCount;
	
	accountArrayCount = [accountArray count];
	
	//Hold on to this array
	originalMenuItemArray = [menuItemArray copy];
	
	//Remove the menu items which we've been passed... we'll be creating our own based off of them.
	[[adium statusController] removeAllMenuItemsForPlugin:self];
	
	//Enumerate our accounts array
	enumerator = [accountArray objectEnumerator];
	while((account = [enumerator nextObject])){    		
		NSMenu			*accountSubmenu = [[[NSMenu allocWithZone:[NSMenu zone]] init] autorelease];
		[accountSubmenu setMenuChangedMessagesEnabled:NO];
		
		//Add status items if we have more than one account
		if(accountArrayCount > 1){
			NSEnumerator	*menuItemEnumerator = [originalMenuItemArray objectEnumerator];
			NSMenuItem		*menuItem;
			NSMenuItem		*accountMenuItem;
			
			//Enumerate all the menu items we were originally passed
			while(menuItem = [menuItemEnumerator nextObject]){
				AIStatus		*status;
				NSDictionary	*newRepresentedObject;
				
				//Set the represented object to indicate both the right status and the right account
				if(status = [[menuItem representedObject] objectForKey:@"AIStatus"]){
					newRepresentedObject = [NSDictionary dictionaryWithObjectsAndKeys:
						status, @"AIStatus",
						account, @"AIAccount",
						nil];
				}else{
					newRepresentedObject = [NSDictionary dictionaryWithObject:account
																	   forKey:@"AIAccount"];
				}
				
				accountMenuItem = [[menuItem copy] autorelease];
				[accountMenuItem setRepresentedObject:newRepresentedObject];
				
				//Add to our menu
				[accountSubmenu addItem:accountMenuItem];
			}
		}

		if([accountSubmenu numberOfItems] > 0){
			[temporaryMenuDict setObject:accountSubmenu
								  forKey:[account internalObjectID]];
			
		}
		
		[accountSubmenu setMenuChangedMessagesEnabled:YES];
	}
	
	//Enumerate all arrays of menu items (for all plugins)
	NSNumber	*identifier;
	
	enumerator = [accountMenuItemArraysDict keyEnumerator];
	while(identifier = [enumerator nextObject]){
		id<AccountMenuPlugin> accountMenuPlugin = [accountMenuPluginsDict objectForKey:identifier];
		
		if([accountMenuPlugin showStatusSubmenu]){
			NSEnumerator	*accountMenuItemEnumerator;
			NSMenuItem		*accountMenuItem;
			
			accountMenuItemArray = [accountMenuItemArraysDict objectForKey:identifier];
			
			//Enumerate each menu item in this array (the array corresponds to one plugin's menu items; each menu item
			//will be for a distinct AIAccount).
			accountMenuItemEnumerator = [accountMenuItemArray objectEnumerator];
			while(accountMenuItem = [accountMenuItemEnumerator nextObject]){
				AIAccount	*account = [accountMenuItem representedObject];
				NSMenu		*generatedAccountSubmenu;
				
				if(generatedAccountSubmenu = [temporaryMenuDict objectForKey:[account internalObjectID]]){
					NSMenu		*accountSubmenu = [generatedAccountSubmenu copy];
					
					//Tell the status controller to update these items as necessary
					[[adium statusController] plugin:self didAddMenuItems:[accountSubmenu itemArray]];
					
					//Set the submenu
					[accountMenuItem setSubmenu:accountSubmenu];
					[accountSubmenu release];
				}
			}
		}
	}
	
	//Clean up
	[originalMenuItemArray release];
}

- (void)dummyAction:(id)sender {};

/*!
 * @brief Remove the passed state menu items
 *
 * When asked to remove state menu items, we remove all submenus off all accounts.  This is much quicker than testing
 * each of the duplicate menu items... this ties the accountController's implementation to the current statusController's
 * implementation, but is worthwhile.
 *
 * @param ignoredMenuItemArray The menu items to remove... ignored in accountController's implementation
 */
- (void)removeStateMenuItems:(NSArray *)ignoredMenuItemArray
{
	NSEnumerator	*enumerator = [accountMenuItemArraysDict objectEnumerator];
	NSArray			*ourMenuItemArray;
	
	while(ourMenuItemArray = [enumerator nextObject]){
		[ourMenuItemArray makeObjectsPerformSelector:@selector(setSubmenu:)
										  withObject:nil];
	}
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)menuItem
{
	return(YES);
}

@end

@implementation AIAccountController (AIAccountControllerObjectSpecifier)
- (NSScriptObjectSpecifier *) objectSpecifier {
	id classDescription = [NSClassDescription classDescriptionForClass:[NSApplication class]];
	NSScriptObjectSpecifier *container = [[NSApplication sharedApplication] objectSpecifier];
	return [[[NSPropertySpecifier alloc] initWithContainerClassDescription:classDescription containerSpecifier:container key:@"accountController"] autorelease];
}
@end
