/* 
 * Adium is the legal property of its developers, whose names are listed in the copyright file included
 * with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU
 * General Public License as published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with this program; if not,
 * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// $Id: AIContentController.m 13406 2005-09-11 19:52:20Z evands $

#import "AIAccountController.h"
#import "AIContactController.h"
#import "ESContactAlertsController.h"
#import "AIContentController.h"
#import "AdiumContentFiltering.h"
#import "AIInterfaceController.h"
#import "AIMenuController.h"

#import "AdiumChatEvents.h"
//XXX - why are we importing specific headers here, rather than just importing the umbrella headers? --boredzo
#import <AIUtilities/AIAttributedStringAdditions.h>
#import <AIUtilities/AIArrayAdditions.h>
#import <AIUtilities/ESImageAdditions.h>
#import <AIUtilities/AIMenuAdditions.h>
#import <AIUtilities/ESExpandedRecursiveLock.h>
#import <AIUtilities/ESImageAdditions.h>
#import <Adium/AIAccount.h>
#import <Adium/AIChat.h>
#import <Adium/AIContentMessage.h>
#import <Adium/AIContentObject.h>
#import <Adium/AIContentStatus.h>
#import <Adium/AIListContact.h>
#import <Adium/AIListGroup.h>
#import <Adium/AIListObject.h>
#import <Adium/AIMetaContact.h>
#import <Adium/NDRunLoopMessenger.h>

@interface AIContentController (PRIVATE)

- (void)finishReceiveContentObject:(AIContentObject *)inObject;
- (void)finishSendContentObject:(AIContentObject *)inObject;
- (void)finishDisplayContentObject:(AIContentObject *)inObject;

- (NSAttributedString *)_filterAttributedString:(NSAttributedString *)inString forContentObject:(AIContentObject *)inObject listObjectContext:(AIListObject *)inListObject usingFilterArray:(NSArray *)inArray;
- (NSString *)_filterString:(NSString *)inString forContentObject:(AIContentObject *)inObject listObjectContext:(AIListObject *)inListObject/* usingFilterArray:(NSArray *)inArray*/;
- (void)_filterContentObject:(AIContentObject *)inObject usingFilterArray:(NSArray *)inArray;
- (NSAttributedString *)thread_filterAttributedString:(NSAttributedString *)attributedString contentFilter:(NSArray *)inContentFilterArray filterContext:(id)filterContext invocation:(NSInvocation *)invocation;
- (NSAttributedString *)_filterAttributedString:(NSAttributedString *)attributedString contentFilter:(NSArray *)inContentFilterArray filterContext:(id)filterContext usingLock:(NSRecursiveLock *)inLock;

- (NSSet *)_informObserversOfChatStatusChange:(AIChat *)inChat withKeys:(NSSet *)modifiedKeys silent:(BOOL)silent;
- (void)chatAttributesChanged:(AIChat *)inChat modifiedKeys:(NSSet *)inModifiedKeys;

@end

/*
 * @class AIContentController
 * @brief Controller to manage incoming and outgoing content and chats.
 *
 * This controller handles default formatting and text entry filters, which can respond as text is entered in a message
 * window.  It the center for content filtering, including registering/unregistering of content filters.
 * It handles sending and receiving of content objects.  It manages chat observers, which are objects notified as
 * status objects are set and removed on AIChat objects.  It manages chats themselves, tracking open ones, closing
 * them when needed, etc.  Finally, it provides Events related to sending and receiving content, such as Message Received.
 */
@implementation AIContentController

/*
 * @brief Initialize the controller
 */
- (void)initController
{
    //Text entry filtering and tracking
    textEntryFilterArray = [[NSMutableArray alloc] init];
    textEntryContentFilterArray = [[NSMutableArray alloc] init];
    textEntryViews = [[NSMutableArray alloc] init];
	chatObserverArray = [[NSMutableArray alloc] init];
    defaultFormattingAttributes = nil;
	emoticonPacks = nil;
	emoticonsArray = nil;
	
    //Chat tracking
    openChats = [[NSMutableSet alloc] init];
	objectsBeingReceived = [[NSMutableSet alloc] init];
	adiumContentFiltering = [[AdiumContentFiltering alloc] init];
	adiumChatEvents = [[AdiumChatEvents alloc] init];	
	[adiumChatEvents controllerDidLoad];	

    //Emoticons array
    emoticonsArray = nil;
	
	//Register the events we generate
	[[adium contactAlertsController] registerEventID:CONTENT_MESSAGE_SENT withHandler:self inGroup:AIMessageEventHandlerGroup globalOnly:NO];
	[[adium contactAlertsController] registerEventID:CONTENT_MESSAGE_RECEIVED withHandler:self inGroup:AIMessageEventHandlerGroup globalOnly:NO];
	[[adium contactAlertsController] registerEventID:CONTENT_MESSAGE_RECEIVED_FIRST withHandler:self inGroup:AIMessageEventHandlerGroup globalOnly:NO];
	[[adium contactAlertsController] registerEventID:CONTENT_MESSAGE_RECEIVED_BACKGROUND withHandler:self inGroup:AIMessageEventHandlerGroup globalOnly:NO];
	
	menuItem_ignore = [[NSMenuItem allocWithZone:[NSMenu menuZone]] initWithTitle:@""
																		   target:self
																		   action:@selector(toggleIgnoreOfContact:)
																	keyEquivalent:@""];
	[[adium menuController] addContextualMenuItem:menuItem_ignore toLocation:Context_Contact_ChatAction];
}

/*
 * @brief Begin closing the controller
 *
 * Post Chat_WillClose for all chats which are still open before the controller closes
 */
- (void)beginClosing
{
	NSEnumerator	*enumerator = [openChats objectEnumerator];
	AIChat			*chat;

	//Every open chat is about to close.
	while((chat = [enumerator nextObject])){
		[[adium notificationCenter] postNotificationName:Chat_WillClose 
												  object:chat
												userInfo:nil];
	}
}

/*
 * @brief Close the controller
 */
- (void)closeController
{
	
}

/*
 * @brief Deallocate
 */
- (void)dealloc
{
	[emoticonPacks release]; emoticonPacks = nil;
	[emoticonsArray release]; emoticonsArray = nil;
    [textEntryFilterArray release];
    [textEntryContentFilterArray release];
    [textEntryViews release];
    [openChats release];
	[chatObserverArray release]; chatObserverArray = nil;
	[objectsBeingReceived release];

    [super dealloc];
}


//Default Formatting -------------------------------------------------------------------------------------------------
#pragma mark Default Formatting
- (void)setDefaultFormattingAttributes:(NSDictionary *)inDict
{
    if(defaultFormattingAttributes != inDict){
	   [defaultFormattingAttributes release];
	   defaultFormattingAttributes	= [inDict retain];
    }
}
/*
 * @brief Default formatting attributes
 *
 * This dictionary of attributes will be used for new text entry views, messages, etc.
 */
- (NSDictionary *)defaultFormattingAttributes
{
	return defaultFormattingAttributes;
}


//Text Entry Filtering -------------------------------------------------------------------------------------------------
#pragma mark Text Entry Filtering

//Text entry filters process content as it is entered by the user.

- (void)registerTextEntryFilter:(id)inFilter
{
	NSParameterAssert([inFilter respondsToSelector:@selector(didOpenTextEntryView:)] &&
					  [inFilter respondsToSelector:@selector(willCloseTextEntryView:)]);
	
	//For performance reasons, we place filters that actually monitor content in a separate array
	if([inFilter respondsToSelector:@selector(stringAdded:toTextEntryView:)] &&
	   [inFilter respondsToSelector:@selector(contentsChangedInTextEntryView:)]){
		[textEntryContentFilterArray addObject:inFilter];
	}else{
		[textEntryFilterArray addObject:inFilter];
	}
}

- (void)unregisterTextEntryFilter:(id)inFilter
{
	[textEntryContentFilterArray removeObject:inFilter];
	[textEntryFilterArray removeObject:inFilter];
}

//A string was added to a text entry view
- (void)stringAdded:(NSString *)inString toTextEntryView:(NSText<AITextEntryView> *)inTextEntryView
{
    NSEnumerator	*enumerator;
    id				filter;
	
    //Notify all text entry filters (that are interested in filtering content)
    enumerator = [textEntryContentFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter stringAdded:inString toTextEntryView:inTextEntryView];
    }
}

//A text entry view's content changed
- (void)contentsChangedInTextEntryView:(NSText<AITextEntryView> *)inTextEntryView
{
    NSEnumerator	*enumerator;
    id				filter;
	
    //Notify all text entry filters (that are interested in filtering content)
    enumerator = [textEntryContentFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter contentsChangedInTextEntryView:inTextEntryView];
    }
}

//A text entry view was opened
- (void)didOpenTextEntryView:(NSText<AITextEntryView> *)inTextEntryView
{
    NSEnumerator	*enumerator;
    id				filter;
	
    //Track the view
    [textEntryViews addObject:inTextEntryView];
    
    //Notify all text entry filters
    enumerator = [textEntryContentFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter didOpenTextEntryView:inTextEntryView];
    }
    enumerator = [textEntryFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter didOpenTextEntryView:inTextEntryView];
    }
}

//A text entry view was closed
- (void)willCloseTextEntryView:(NSText<AITextEntryView> *)inTextEntryView
{
    NSEnumerator	*enumerator;
    id				filter;
	
    //Stop tracking the view
    [textEntryViews removeObject:inTextEntryView];
	
    //Notify all text entry filters
    enumerator = [textEntryContentFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter willCloseTextEntryView:inTextEntryView];
    }
    enumerator = [textEntryFilterArray objectEnumerator];
    while((filter = [enumerator nextObject])){
        [filter willCloseTextEntryView:inTextEntryView];
    }
}

//Returns all currently open text entry views
- (NSArray *)openTextEntryViews
{
    return(textEntryViews);
}


//Content Filtering ----------------------------------------------------------------------------------------------------
#pragma mark Content Filtering
- (void)registerContentFilter:(id <AIContentFilter>)inFilter
					   ofType:(AIFilterType)type
					direction:(AIFilterDirection)direction {
	[adiumContentFiltering registerContentFilter:inFilter ofType:type direction:direction];
}
- (void)registerDelayedContentFilter:(id <AIDelayedContentFilter>)inFilter
							  ofType:(AIFilterType)type
						   direction:(AIFilterDirection)direction {
	[adiumContentFiltering registerDelayedContentFilter:inFilter ofType:type direction:direction];
}
- (void)unregisterContentFilter:(id <AIContentFilter>)inFilter {
	[adiumContentFiltering unregisterContentFilter:inFilter];
}
- (void)registerFilterStringWhichRequiresPolling:(NSString *)inPollString {
	[adiumContentFiltering registerFilterStringWhichRequiresPolling:inPollString];
}
- (BOOL)shouldPollToUpdateString:(NSString *)inString {
	return [adiumContentFiltering shouldPollToUpdateString:inString];
}
- (NSAttributedString *)filterAttributedString:(NSAttributedString *)attributedString
							   usingFilterType:(AIFilterType)type
									 direction:(AIFilterDirection)direction
									   context:(id)context {
	return [adiumContentFiltering filterAttributedString:attributedString
										 usingFilterType:type
											   direction:direction
												 context:context];
}
- (void)filterAttributedString:(NSAttributedString *)attributedString
			   usingFilterType:(AIFilterType)type
					 direction:(AIFilterDirection)direction
				 filterContext:(id)filterContext
			   notifyingTarget:(id)target
					  selector:(SEL)selector
					   context:(id)context {
	[adiumContentFiltering filterAttributedString:attributedString
								  usingFilterType:type
										direction:direction
									filterContext:filterContext
								  notifyingTarget:target
										 selector:selector
										  context:context];
}
- (void)delayedFilterDidFinish:(NSAttributedString *)attributedString uniqueID:(unsigned long long)uniqueID
{
	[adiumContentFiltering delayedFilterDidFinish:attributedString
										 uniqueID:uniqueID];
}

//Messaging ------------------------------------------------------------------------------------------------------------
#pragma mark Messaging
//Receiving step 1: Add an incoming content object - entry point
- (void)receiveContentObject:(AIContentObject *)inObject
{
	if (inObject) {
		AIChat			*chat = [inObject chat];

		//Only proceed if the contact is not ignored
		if (![chat isListContactIgnored:[inObject source]]) {
			//Notify: Will Receive Content
			if ([inObject trackContent]) {
				[[adium notificationCenter] postNotificationName:Content_WillReceiveContent
														  object:chat
														userInfo:[NSDictionary dictionaryWithObjectsAndKeys:inObject,@"Object",nil]];
			}

			//Track that we are in the process of receiving this object
			[objectsBeingReceived addObject:inObject];
			
			//Run the object through our incoming content filters
			if ([inObject filterContent]) {
				[self filterAttributedString:[inObject message]
							 usingFilterType:AIFilterContent
								   direction:AIFilterIncoming
							   filterContext:inObject
							 notifyingTarget:self
									selector:@selector(didFilterAttributedString:receivingContext:)
									 context:inObject];
				
			} else {
				[self finishReceiveContentObject:inObject];
			}
		}
    }
}

//Receiving step 2: filtering callback
- (void)didFilterAttributedString:(NSAttributedString *)filteredMessage receivingContext:(AIContentObject *)inObject
{
	[inObject setMessage:filteredMessage];
	
	[self finishReceiveContentObject:inObject];
}

//Receiving step 3: Display the content
- (void)finishReceiveContentObject:(AIContentObject *)inObject
{
	//Display the content
	[self displayContentObject:inObject];
	
	//Update our most recent chat
	if([inObject trackContent]){
		AIChat	*chat = [inObject chat];
		
		if(chat != mostRecentChat){
			[mostRecentChat release];
			mostRecentChat = [chat retain];
		}
	}
}

//Sending step 1: Entry point for any method in Adium which sends content
- (BOOL)sendContentObject:(AIContentObject *)inObject
{
    //Run the object through our outgoing content filters
    if([inObject filterContent]){
		[self filterAttributedString:[inObject message]
					 usingFilterType:AIFilterContent
						   direction:AIFilterOutgoing
					   filterContext:inObject
					 notifyingTarget:self
							selector:@selector(didFilterAttributedString:contentSendingContext:)
							 context:inObject];
		
    }else{
		[self finishSendContentObject:inObject];
	}
	
	// XXX
	return YES;
}

//Sending step 2: Sending filter callback
-(void)didFilterAttributedString:(NSAttributedString *)filteredString contentSendingContext:(AIContentObject *)inObject
{
	[inObject setMessage:filteredString];

	//Special outgoing content filter for AIM away message bouncing.  Used to filter %n,%t,...
	if([inObject isKindOfClass:[AIContentMessage class]] && [(AIContentMessage *)inObject isAutoreply]){
		[self filterAttributedString:[inObject message]
					 usingFilterType:AIFilterAutoReplyContent
						   direction:AIFilterOutgoing
					   filterContext:inObject
					 notifyingTarget:self
							selector:@selector(didFilterAttributedString:autoreplySendingContext:)
							 context:inObject];
	}else{		
		[self finishSendContentObject:inObject];
	}
}

//Sending step 3, applicable only when sending an autreply: Filter callback
-(void)didFilterAttributedString:(NSAttributedString *)filteredString autoreplySendingContext:(AIContentObject *)inObject
{
	[inObject setMessage:filteredString];

	[self finishSendContentObject:inObject];
}

//Sending step 4: Post notifications and ask the account to actually send the content.
//The account is responsible for invoking step 5 which will lead to the actual display.
- (void)finishSendContentObject:(AIContentObject *)inObject
{
    AIChat		*chat = [inObject chat];
	
	//Notify: Will Send Content
    if([inObject trackContent]){
        [[adium notificationCenter] postNotificationName:Content_WillSendContent
												  object:chat 
												userInfo:[NSDictionary dictionaryWithObjectsAndKeys:inObject,@"Object",nil]];
    }
	
    //Send the object
	if ([inObject sendContent]){
		if([(AIAccount *)[inObject source] sendContentObject:inObject]){
			if([inObject displayContent]){
				//Add the object
				[self displayContentObject:inObject];
			}
			
			if([inObject trackContent]){
				//Did send content
				[[adium contactAlertsController] generateEvent:CONTENT_MESSAGE_SENT
												 forListObject:[chat listObject]
													  userInfo:[NSDictionary dictionaryWithObjectsAndKeys:chat,@"AIChat",inObject,@"AIContentObject",nil]
								  previouslyPerformedActionIDs:nil];				
			}
			
			if(mostRecentChat != chat){
				[mostRecentChat release];
				mostRecentChat = [chat retain];
			}
//			sent = YES;
		}
	}else{
		//We shouldn't send the content, so something was done with it.. clear the text entry view
		//XXX - Nobody is observing this notification... -ai
		[[adium notificationCenter] postNotificationName:Interface_ShouldClearTextEntryView
												  object:chat 
												userInfo:nil];
	}
	
//    return(sent);
}

//Display a content object
//Add content to the message view.  Doesn't do any sending or receiving, just adds the content.
- (void)displayContentObject:(AIContentObject *)inObject usingContentFilters:(BOOL)useContentFilters
{
	[self displayContentObject:inObject usingContentFilters:useContentFilters immediately:NO];
}

//Immediately YES means the main thread will halt until the content object is displayed;
//Immediately NO shuffles it off into the filtering thread, which will handle content sequentially but allows the main
//thread to continue operation.  
//This facility primarily exists for message history, which needs to put its display in before the first message;
//without this, the use of threaded filtering means that message history shows up after the first message.
- (void)displayContentObject:(AIContentObject *)inObject usingContentFilters:(BOOL)useContentFilters immediately:(BOOL)immediately
{
	if (useContentFilters){
		
		if (immediately){
			//Filter in the main thread, set the message, and continue
			[inObject setMessage:[self filterAttributedString:[inObject message]
											  usingFilterType:AIFilterContent
													direction:([inObject isOutgoing] ? AIFilterOutgoing : AIFilterIncoming)
													  context:inObject]];
			[self displayContentObject:inObject immediately:YES];
			
			
		}else{
			//Filter in the filter thread
			[self filterAttributedString:[inObject message]
						 usingFilterType:AIFilterContent
							   direction:([inObject isOutgoing] ? AIFilterOutgoing : AIFilterIncoming)
						   filterContext:inObject
						 notifyingTarget:self
								selector:@selector(didFilterAttributedString:contentFilterDisplayContext:)
								 context:inObject];
		}
	}else{
		//Just continue
		[self displayContentObject:inObject immediately:immediately];
	}
}

- (void)didFilterAttributedString:(NSAttributedString *)filteredString contentFilterDisplayContext:(AIContentObject *)inObject
{
	[inObject setMessage:filteredString];
	
	//Continue
	[self displayContentObject:inObject immediately:NO];
}

//Display a content object
//Add content to the message view.  Doesn't do any sending or receiving, just adds the content.
- (void)displayContentObject:(AIContentObject *)inObject
{
	[self displayContentObject:inObject immediately:NO];
}

- (void)displayContentObject:(AIContentObject *)inObject immediately:(BOOL)immediately
{
    //Filter the content object
    if([inObject filterContent]){
		BOOL				message = ([inObject isKindOfClass:[AIContentMessage class]] && ![(AIContentMessage *)inObject isAutoreply]);
		AIFilterType		filterType = (message ? AIFilterMessageDisplay : AIFilterDisplay);
		AIFilterDirection	direction = ([inObject isOutgoing] ? AIFilterOutgoing : AIFilterIncoming);
		
		if (immediately){
			
			//Set it after filtering in the main thread, then display it
			[inObject setMessage:[self filterAttributedString:[inObject message]
											  usingFilterType:filterType
													direction:direction
													  context:inObject]];
			[self finishDisplayContentObject:inObject];		
			
		}else{
			//Filter in the filtering thread
			[self filterAttributedString:[inObject message]
						 usingFilterType:filterType
							   direction:direction
						   filterContext:inObject
						 notifyingTarget:self
								selector:@selector(didFilterAttributedString:displayContext:)
								 context:inObject];
		}
		
    }else{
		[self finishDisplayContentObject:inObject];
	}

}

- (void)didFilterAttributedString:(NSAttributedString *)filteredString displayContext:(AIContentObject *)inObject
{
	[inObject setMessage:filteredString];
	
	[self finishDisplayContentObject:inObject];
}

- (void)finishDisplayContentObject:(AIContentObject *)inObject
{
    //Check if the object should display
    if([inObject displayContent] && ([[inObject message] length] > 0)){
		AIChat			*chat = [inObject chat];
		NSDictionary	*userInfo;
		BOOL			contentReceived, shouldPostContentReceivedEvents, chatIsOpen;

		//If the chat of the content object has been cleared, we can't do anything with it, so simply return
		if(!chat) return;
		
		chatIsOpen = [chat isOpen];
		contentReceived = (([inObject isMemberOfClass:[AIContentMessage class]]) &&
						   (![inObject isOutgoing]));
		shouldPostContentReceivedEvents = contentReceived && [inObject trackContent];
		
		if(!chatIsOpen){
			/*
			 Tell the interface to open the chat
			 For incoming messages, we don't open the chat until we're sure that new content is being received.
			 */
			[[adium interfaceController] openChat:chat];
		}

		userInfo = [NSDictionary dictionaryWithObjectsAndKeys:chat, @"AIChat", inObject, @"AIContentObject", nil];

		//Add this content to the chat
		[chat addContentObject:inObject];
		
		//Notify: Content Object Added
		[[adium notificationCenter] postNotificationName:Content_ContentObjectAdded
												  object:chat
												userInfo:userInfo];

		if(shouldPostContentReceivedEvents){
			NSSet			*previouslyPerformedActionIDs = nil;
			AIListObject	*listObject = [chat listObject];
			
			if(!chatIsOpen){
				//If the chat wasn't open before, generate CONTENT_MESSAGE_RECEIVED_FIRST
				previouslyPerformedActionIDs = [[adium contactAlertsController] generateEvent:CONTENT_MESSAGE_RECEIVED_FIRST
																				forListObject:listObject
																					 userInfo:userInfo
																 previouslyPerformedActionIDs:nil];	
			}
			
			if(chat != [[adium interfaceController] activeChat]){
				//If the chat is not currently active, generate CONTENT_MESSAGE_RECEIVED_BACKGROUND
				previouslyPerformedActionIDs = [[adium contactAlertsController] generateEvent:CONTENT_MESSAGE_RECEIVED_BACKGROUND
																				forListObject:listObject
																					 userInfo:userInfo
																 previouslyPerformedActionIDs:previouslyPerformedActionIDs];
			}
			
			[[adium contactAlertsController] generateEvent:CONTENT_MESSAGE_RECEIVED
											 forListObject:listObject
												  userInfo:userInfo
							  previouslyPerformedActionIDs:previouslyPerformedActionIDs];
		}
    }

	//We are no longer in the process of receiving this object
	[objectsBeingReceived removeObject:inObject];
	AILog(@"objectsBeingReceived: %@",([objectsBeingReceived count] ? [objectsBeingReceived description] : @"(empty)"));
}

- (void)displayStatusMessage:(NSString *)message ofType:(NSString *)type inChat:(AIChat *)inChat
{
	AIContentStatus		*content;
	NSAttributedString	*attributedMessage;
	
	//Create our content object
	attributedMessage = [[NSAttributedString alloc] initWithString:message
														attributes:[self defaultFormattingAttributes]];
	content = [AIContentStatus statusInChat:inChat
								 withSource:[inChat listObject]
								destination:[inChat account]
									   date:[NSDate date]
									message:attributedMessage
								   withType:type];
	[attributedMessage release];

	//Add the object
	[self receiveContentObject:content];
}

//Returns YES if the account/chat is available for sending content
- (BOOL)availableForSendingContentType:(NSString *)inType toContact:(AIListContact *)inContact onAccount:(AIAccount *)inAccount 
{
	return([inAccount availableForSendingContentType:inType toContact:inContact]);
}



//Chat Status -------------------------------------------------------------------------------------------------
#pragma mark Chat Status

//Registers code to observe handle status changes
- (void)registerChatObserver:(id <AIChatObserver>)inObserver
{
	//Add the observer
    [chatObserverArray addObject:[NSValue valueWithNonretainedObject:inObserver]];
	
    //Let the new observer process all existing chats
	[self updateAllChatsForObserver:inObserver];
}

- (void)unregisterChatObserver:(id <AIChatObserver>)inObserver
{
    [chatObserverArray removeObject:[NSValue valueWithNonretainedObject:inObserver]];
}

- (void)chatStatusChanged:(AIChat *)inChat modifiedStatusKeys:(NSSet *)inModifiedKeys silent:(BOOL)silent
{
	NSSet			*modifiedAttributeKeys;
	
    //Let all observers know the contact's status has changed before performing any further notifications
	modifiedAttributeKeys = [self _informObserversOfChatStatusChange:inChat withKeys:inModifiedKeys silent:silent];

    //Post an attributes changed message (if necessary)
    if([modifiedAttributeKeys count]){
		[self chatAttributesChanged:inChat modifiedKeys:modifiedAttributeKeys];
    }	
}

- (void)chatAttributesChanged:(AIChat *)inChat modifiedKeys:(NSSet *)inModifiedKeys
{
	//Post an attributes changed message
	[[adium notificationCenter] postNotificationName:Chat_AttributesChanged
											  object:inChat
											userInfo:(inModifiedKeys ? [NSDictionary dictionaryWithObject:inModifiedKeys 
																								   forKey:@"Keys"] : nil)];
}

//Send a chatStatusChanged message for each open chat with a nil modifiedStatusKeys array
- (void)updateAllChatsForObserver:(id <AIChatObserver>)observer
{
	NSEnumerator	*enumerator = [openChats objectEnumerator];
	AIChat			*chat;
	
	while (chat = [enumerator nextObject]){
		[self chatStatusChanged:chat modifiedStatusKeys:nil silent:NO];
	}
}

//Notify observers of a status change.  Returns the modified attribute keys
- (NSSet *)_informObserversOfChatStatusChange:(AIChat *)inChat withKeys:(NSSet *)modifiedKeys silent:(BOOL)silent
{
	NSMutableSet	*attrChange = nil;
	NSEnumerator	*enumerator;
	NSValue			*observerValue;

	//Let our observers know
	enumerator = [chatObserverArray objectEnumerator];
	while((observerValue = [enumerator nextObject])){
		id <AIChatObserver>	observer;
		NSSet				*newKeys;
		
		observer = [observerValue nonretainedObjectValue];
		if(newKeys = [observer updateChat:inChat keys:modifiedKeys silent:silent]){
			if (!attrChange) attrChange = [NSMutableSet set];
			[attrChange unionSet:newKeys];
		}
	}
	
	//Send out the notification for other observers
	[[adium notificationCenter] postNotificationName:Chat_StatusChanged
											  object:inChat
											userInfo:(modifiedKeys ? [NSDictionary dictionaryWithObject:modifiedKeys 
																								 forKey:@"Keys"] : nil)];
	
	return(attrChange);
}

//Increase unviewed content
- (void)increaseUnviewedContentOfChat:(AIChat *)inChat
{
	int currentUnviewed = [inChat integerStatusObjectForKey:KEY_UNVIEWED_CONTENT];
	[inChat setStatusObject:[NSNumber numberWithInt:(currentUnviewed+1)]
					 forKey:KEY_UNVIEWED_CONTENT
					 notify:YES];
}

//Clear unviewed content
- (void)clearUnviewedContentOfChat:(AIChat *)inChat
{
	[inChat setStatusObject:nil forKey:KEY_UNVIEWED_CONTENT notify:YES];
}

//Chats -------------------------------------------------------------------------------------------------
#pragma mark Chats
//Opens a chat for communication with the contact, creating if necessary.  The new chat will be made active.
- (AIChat *)openChatWithContact:(AIListContact *)inContact
{
	AIChat	*chat = [self chatWithContact:inContact];

	if(chat) [[adium interfaceController] openChat:chat]; 

	return(chat);	
}

//Creates a chat for communication with the contact, but does not make the chat active (Doesn't open a chat window)
//If a chat already exists it will be returned
- (AIChat *)chatWithContact:(AIListContact *)inContact
{
	NSEnumerator	*enumerator;
	AIChat			*chat = nil;
	AIListContact	*targetContact = inContact;
		
	/*
	 If we're dealing with a meta contact, open a chat with the preferred contact for this meta contact
	 It's a good idea for the caller to pick the preferred contact for us, since they know the content type
	 being sent and more information - but we'll do it here as well just to be safe.
	 */
	if ([inContact isKindOfClass:[AIMetaContact class]]){
		targetContact = [[adium contactController] preferredContactForContentType:CONTENT_MESSAGE_TYPE
																   forListContact:inContact];
		
		/*
		 If we have no accounts online, preferredContactForContentType:forListContact will return nil.
		 We'd rather open up the chat window on a useless contact than do nothing, so just pick the 
		 preferredContact from the metaContact.
		 */
		if (!targetContact){
			targetContact = [(AIMetaContact *)inContact preferredContact];
		}
	}
	
	//If we can't get a contact, we're not going to be able to get a chat... return nil
	if(!targetContact) return nil;
	
	//Search for an existing chat we can switch instead of replacing
	enumerator = [openChats objectEnumerator];
	while(chat = [enumerator nextObject]){
		//If a chat for this object already exists
		if([[chat uniqueChatID] isEqualToString:[targetContact internalObjectID]]) {
			if(!([chat listObject] == targetContact)){
				[self switchChat:chat toAccount:[targetContact account]];
			}
			
			break;
		}
		
		//If this object is within a meta contact, and a chat for an object in that meta contact already exists
		if([[targetContact containingObject] isKindOfClass:[AIMetaContact class]] && 
		   [[chat listObject] containingObject] == [targetContact containingObject]){

			//Switch the chat to be on this contact (and its account) now
			[self switchChat:chat toListContact:targetContact usingContactAccount:YES];
			
			break;
		}
	}

	if(!chat){
		AIAccount	*account;
		account = [targetContact account];
		
		//Create a new chat
		chat = [AIChat chatForAccount:account];
		[chat addParticipatingListObject:targetContact];
		[openChats addObject:chat];
		AILog(@"chatWithContact: Added <<%@>> [%@]",chat,openChats);

		//Inform the account of its creation and post a notification if successful
		if([[targetContact account] openChat:chat]){
			[[adium notificationCenter] postNotificationName:Chat_Created object:chat userInfo:nil];
		}else{
			[openChats removeObject:chat];
			AILog(@"chatWithContact: Immediately removed <<%@>> [%@]",chat,openChats);
			chat = nil;
		}
	}

	return(chat);
}

/*!
* @brief Return a pre-existing chat with a contact.
 *
 * @result The chat, or nil if no chat with the contact exists
 */
- (AIChat *)existingChatWithContact:(AIListContact *)inContact
{
	NSEnumerator	*enumerator;
	AIChat			*chat = nil;
	
	if ([inContact isKindOfClass:[AIMetaContact class]]) {
		//Search for a chat with any contact within this AIMetaContact
		enumerator = [openChats objectEnumerator];
		while ((chat = [enumerator nextObject])) {
			if ([[(AIMetaContact *)inContact containedObjects] containsObjectIdenticalTo:[chat listObject]]) break;
		}
		
	} else {
		//Search for a chat with this AIListContact
		enumerator = [openChats objectEnumerator];
		while ((chat = [enumerator nextObject])) {
			if ([chat listObject] == inContact) break;
		}
	}
	
	return chat;
}

- (AIChat *)chatWithName:(NSString *)inName onAccount:(AIAccount *)account chatCreationInfo:(NSDictionary *)chatCreationInfo
{
	AIChat			*chat = nil;
	
	//Search for an existing chat we can use instead of creating a new one
	chat = [self existingChatWithName:inName onAccount:account];
	
	if(!chat){
		//Create a new chat
		chat = [AIChat chatForAccount:account];
		[chat setName:inName];
		[openChats addObject:chat];
		AILog(@"chatWithName:%@ onAccount:%@ added <<%@>> [%@]",inName,account,chat,openChats);
		
		if (chatCreationInfo) [chat setStatusObject:chatCreationInfo
											 forKey:@"ChatCreationInfo"
											 notify:NotifyNever];
		
		[chat setStatusObject:[NSNumber numberWithBool:YES]
					   forKey:@"AlwaysShowUserList"
					   notify:NotifyNever];
		
		//Inform the account of its creation and post a notification if successful
		if([account openChat:chat]){
			[[adium notificationCenter] postNotificationName:Chat_Created object:chat userInfo:nil];
		}else{
			[openChats removeObject:chat];
			AILog(@"chatWithName: Immediately removed <<%@>> [%@]",chat,openChats);
			chat = nil;
		}
	}
	return(chat);
}

- (void)openChat:(AIChat *)chat
{
	if(chat){		
		[openChats addObject:chat];
		AILog(@"openChat: Added <<%@>> [%@]",chat,openChats);
		[[adium interfaceController] openChat:chat]; 
	}
}

- (AIChat *)existingChatWithName:(NSString *)inName onAccount:(AIAccount *)account
{
	NSEnumerator	*enumerator;
	AIChat			*chat = nil;
	
	enumerator = [openChats objectEnumerator];

	while(chat = [enumerator nextObject]){
		if(([chat account] == account) &&
		   ([[chat name] isEqualToString:inName])){
			break;
		}
	}	
	
	return chat;
}

- (AIChat *)existingChatWithUniqueChatID:(NSString *)uniqueChatID
{
	NSEnumerator	*enumerator;
	AIChat			*chat = nil;
	
	enumerator = [openChats objectEnumerator];
	
	while(chat = [enumerator nextObject]){
		if([[chat uniqueChatID] isEqualToString:uniqueChatID]){
			break;
		}
	}	
	
	return chat;
}

//Close a chat
- (BOOL)closeChat:(AIChat *)inChat
{	
	BOOL	shouldRemove = YES;
	
	/* If we are currently passing a content object for this chat through our content filters, don't remove it from
	 * our openChats set as it will become needed soon. If we were to remove it, and a second message came in which was
	 * also before the first message is done filtering, we would otherwise mistakenly think we needed to create a new
	 * chat, generating a duplicate.
	 */
	NSEnumerator	*objectsBeingReceivedEnumerator = [objectsBeingReceived objectEnumerator];
	AIContentObject	*contentObject;
	while(contentObject = [objectsBeingReceivedEnumerator nextObject]){
		if([contentObject chat] == inChat){
			shouldRemove = NO;
			break;
		}
	}

	if(mostRecentChat == inChat){
		[mostRecentChat release];
		mostRecentChat = nil;
	}
	
	//Notify the account and send out the Chat_WillClose notification
	[[inChat account] closeChat:inChat];
	[[adium notificationCenter] postNotificationName:Chat_WillClose object:inChat userInfo:nil];
	
	//Remove the chat's content (it retains the chat, so this must be done separately)
	[inChat removeAllContent];

	//Remove the chat
	if(shouldRemove){
		[openChats removeObject:inChat];
		AILog(@"closeChat: Removed <<%@>> [%@]",inChat, openChats);
	}

	return YES;
}

//Switch a chat from one account to another, updating the target list contact to be an 'identical' one on the target account.
- (void)switchChat:(AIChat *)chat toAccount:(AIAccount *)newAccount
{
	AIAccount	*oldAccount = [chat account];
	if (newAccount != oldAccount){
		//Hang onto stuff until we're done
		[chat retain];

		//Close down the chat on account A
		[oldAccount closeChat:chat];

		//Set the account and the listObject
		{
			[chat setAccount:newAccount];
			
			//We want to keep the same destination for the chat but switch it to a listContact on the desired account.
			AIListContact	*newContact = [[adium contactController] contactWithService:[[chat listObject] service]
																				account:[chat account]
																					UID:[[chat listObject] UID]];
			[chat setListObject:newContact];
		}
		
		//Open the chat on account B
		[newAccount openChat:chat];
		
		//Clean up
		[chat release];
	}
}

//Switch the list contact of the account; this does not change the source account - use switchChat:toAccount: for that.
- (void)switchChat:(AIChat *)chat toListContact:(AIListContact *)inContact usingContactAccount:(BOOL)useContactAccount
{
	AIAccount		*newAccount = (useContactAccount ? [inContact account] : [chat account]);
	
	//Switch the inContact over to a contact on the new account so we send messages to the right place.
	AIListContact	*newContact = [[adium contactController] contactWithService:[inContact service]
																		account:newAccount
																			UID:[inContact UID]];
	if (newContact != [chat listObject]){
		//Hang onto stuff until we're done
		[chat retain];
		
		//Close down the chat on the account, as the account may need to perform actions such as closing a connection
		[[chat account] closeChat:chat];
		
		//Set to the new listContact and account as needed
		[chat setListObject:newContact];
		if (useContactAccount) [chat setAccount:newAccount];
		
		
		//Reopen the chat on the account
		[[chat account] openChat:chat];
		
		//Clean up
		[chat release];
	}
}

//Returns all chats with the object
- (NSSet *)allChatsWithContact:(AIListContact *)inContact
{
    NSMutableSet	*foundChats = nil;
	
	//Scan the objects participating in each chat, looking for the requested object
	if([inContact isKindOfClass:[AIMetaContact class]]){

		NSEnumerator	*enumerator;
		AIListContact	*listContact;

		foundChats = [NSMutableSet set];
		
		enumerator = [[(AIMetaContact *)inContact listContacts] objectEnumerator];
		while(listContact = [enumerator nextObject]){
			NSSet		*listContactChats;
			if (listContactChats = [self allChatsWithContact:listContact]){
				[foundChats unionSet:listContactChats];
			}
		}
		
	}else{
		NSEnumerator	*chatEnumerator = [openChats objectEnumerator];
		AIChat			*chat;
		while((chat = [chatEnumerator nextObject])){
			if(![chat name] &&
				[[[chat listObject] internalObjectID] isEqualToString:[inContact internalObjectID]]){
				if (!foundChats) foundChats = [NSMutableSet set];
				[foundChats addObject:chat];
			}
		}
	}
	
    return(foundChats);
}

- (NSSet *)openChats
{
    return openChats;
}

//Switch to a chat with the most recent unviewed content.  Returns YES if one existed
- (AIChat *)mostRecentUnviewedChat
{
	AIChat  *newActiveChat = nil;

    if(mostRecentChat && [mostRecentChat integerStatusObjectForKey:KEY_UNVIEWED_CONTENT]){
		//First choice: switch to the chat which received chat most recently if it has unviewed content
		newActiveChat = mostRecentChat;
		
	}else{
		//Second choice: switch to the first chat we can find which has unviewed content
		NSEnumerator	*enumerator = [openChats objectEnumerator];
		AIChat			*chat;
		while ((chat = [enumerator nextObject]) && ![chat integerStatusObjectForKey:KEY_UNVIEWED_CONTENT]);
		
		if (chat) newActiveChat = chat;
	}

	return newActiveChat;
}

/*!
 * @brief Is the passed contact in a group chat?
 *
 * @result YES if the contact is in an open group chat; NO if not.
 */
- (BOOL)contactIsInGroupChat:(AIListContact *)listContact
{
	NSEnumerator	*chatEnumerator = [openChats objectEnumerator];
	AIChat			*chat;
	BOOL			contactIsInGroupChat = NO;

	while((chat = [chatEnumerator nextObject])){
		if([chat name] &&
		   [[chat participatingListObjects] containsObjectIdenticalTo:listContact]){

			contactIsInGroupChat = YES;
			break;
		}
	}
	
	return contactIsInGroupChat;
}

//Content Source & Destination -----------------------------------------------------------------------------------------
#pragma mark Content Source & Destination
//Returns the available account for sending content to a specified contact
- (NSArray *)sourceAccountsForSendingContentType:(NSString *)inType
									toListObject:(AIListObject *)inObject
									   preferred:(BOOL)inPreferred
{
	NSMutableArray	*sourceAccounts = [NSMutableArray array];
	NSEnumerator	*enumerator = [[[adium accountController] accountArray] objectEnumerator];
	AIAccount		*account;
	
	while(account = [enumerator nextObject]){
		if([inObject service] == [account service]){
			BOOL			knowsObject = NO;
			BOOL			couldSendContent = NO;
			AIListContact	*contactForAccount = [[adium contactController] existingContactWithService:[inObject service]
																							   account:account
																								   UID:[inObject UID]];
			//Does the account know this object?
			if(contactForAccount){
				knowsObject = [account availableForSendingContentType:inType toContact:contactForAccount];
			}
			
			//Could the account send this
			couldSendContent = [account availableForSendingContentType:inType toContact:nil];
			
			if((inPreferred && knowsObject) || (!inPreferred && !knowsObject && couldSendContent)){
				[sourceAccounts addObject:account];
			}
		}
	}
	
	return(sourceAccounts);
}

//Returns the available contacts for receiving content to a specific contact
- (NSArray *)destinationObjectsForContentType:(NSString *)inType
								 toListObject:(AIListObject *)inObject
									preferred:(BOOL)inPreferred
{
	//meta contact special case here, return any contacts in the user defined meta contact
	return([NSArray arrayWithObject:inObject]);
}

#pragma mark Contact Alerts

- (NSString *)shortDescriptionForEventID:(NSString *)eventID
{
	NSString	*description;
	
	if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
		description = AILocalizedString(@"Is sent a message",nil);
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED]){
		description = AILocalizedString(@"Sends a message",nil);
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST]){
		description = AILocalizedString(@"Sends an initial message",nil);
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
		description = AILocalizedString(@"Sends a message in a background chat",nil);
	}else{
		description = @"";
	}
	
	return(description);
}

- (NSString *)globalShortDescriptionForEventID:(NSString *)eventID
{
	NSString	*description;

	if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
		description = AILocalizedString(@"Message sent",nil);
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED]){
		description = AILocalizedString(@"Message received",nil);
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST]){
		description = AILocalizedString(@"Message received (Initial)",nil);
	}else if ([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
		description = AILocalizedString(@"Message received (Background chat)",nil);
	}else{
		description = @"";
	}

	return(description);
}

//Evan: This exists because old X(tras) relied upon matching the description of event IDs, and I don't feel like making
//a converter for old packs.  If anyone wants to fix this situation, please feel free :)
- (NSString *)englishGlobalShortDescriptionForEventID:(NSString *)eventID
{
	NSString	*description;
	
	if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
		description = @"Message Sent";
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED]){
		description = @"Message Received";
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST]){
		description = @"Message Received (New)";
	}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
		description = @"Message Received (Background Chat)";
	}else{
		description = @"";
	}
	
	return(description);
}

- (NSString *)longDescriptionForEventID:(NSString *)eventID forListObject:(AIListObject *)listObject
{
	NSString	*description = nil;
	
	if(listObject){
		NSString	*name;
		NSString	*format;

		if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
			format = AILocalizedString(@"When you send %@ a message",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED]){
			format = AILocalizedString(@"When %@ sends a message to you",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST]){
			format = AILocalizedString(@"When %@ sends an initial message to you",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
			format = AILocalizedString(@"When %@ sends a message to you in a background chat",nil);			
		}else{
			format = nil;
		}

		if(format){
			name = ([listObject isKindOfClass:[AIListGroup class]] ?
					[NSString stringWithFormat:AILocalizedString(@"a member of %@",nil),[listObject displayName]] :
					[listObject displayName]);
			
			description = [NSString stringWithFormat:format, name];
		}
		
	}else{
		if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
			description = AILocalizedString(@"When you send a message",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED]){
			description = AILocalizedString(@"When you receive any message",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST]){
			description = AILocalizedString(@"When you receive an initial message",nil);
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
			description = AILocalizedString(@"When you receive a message in a background chat",nil);			
		}
	}
	
	return(description);
}

- (NSString *)naturalLanguageDescriptionForEventID:(NSString *)eventID
										listObject:(AIListObject *)listObject
										  userInfo:(id)userInfo
									includeSubject:(BOOL)includeSubject
{
	NSString		*description = nil;
	AIContentObject	*contentObject;
	NSString		*messageText;
	NSString		*displayName;
	
	NSParameterAssert([userInfo isKindOfClass:[NSDictionary class]]);
	
	contentObject = [(NSDictionary *)userInfo objectForKey:@"AIContentObject"];
	messageText = [[[contentObject message] attributedStringByConvertingAttachmentsToStrings] string];
	
	if(includeSubject){
		
		if([eventID isEqualToString:CONTENT_MESSAGE_SENT]){
			displayName = (listObject ? [listObject displayName] : [[contentObject chat] name]);

			description = [NSString stringWithFormat:
				AILocalizedString(@"You said %@ to %@","You said Message to Contact"),
				messageText,
				displayName];
			
		}else if([eventID isEqualToString:CONTENT_MESSAGE_RECEIVED] ||
				 [eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_FIRST] ||
				 [eventID isEqualToString:CONTENT_MESSAGE_RECEIVED_BACKGROUND]){
			displayName = (listObject ? [listObject displayName] : [[contentObject source] displayName]);

			description = [NSString stringWithFormat:
				AILocalizedString(@"%@ said %@","Contact said Message"),
				displayName,
				messageText];
		}	
		
	}else{
		description = messageText;
	}
	
	return(description);
}

/*! 
* @brief Generate a menu of encryption preference choices
*/
- (NSMenu *)encryptionMenuNotifyingTarget:(id)target withDefault:(BOOL)withDefault
{
	NSMenu		*encryptionMenu = [[NSMenu allocWithZone:[NSMenu zone]] init];
	NSMenuItem	*menuItem;

	[encryptionMenu setTitle:ENCRYPTION_MENU_TITLE];

	menuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Disable chat encryption",nil)
										  target:target
										  action:@selector(selectedEncryptionPreference:)
								   keyEquivalent:@""];
	
	[menuItem setTag:EncryptedChat_Never];
	[encryptionMenu addItem:menuItem];
	[menuItem release];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Encrypt chats as requested",nil)
										  target:target
										  action:@selector(selectedEncryptionPreference:)
								   keyEquivalent:@""];
	
	[menuItem setTag:EncryptedChat_Manually];
	[encryptionMenu addItem:menuItem];
	[menuItem release];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Encrypt chats automatically",nil)
										  target:target
										  action:@selector(selectedEncryptionPreference:)
								   keyEquivalent:@""];
	
	[menuItem setTag:EncryptedChat_Automatically];
	[encryptionMenu addItem:menuItem];
	[menuItem release];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Force encryption and refuse plaintext",nil)
										  target:target
										  action:@selector(selectedEncryptionPreference:)
								   keyEquivalent:@""];
	
	[menuItem setTag:EncryptedChat_RejectUnencryptedMessages];
	[encryptionMenu addItem:menuItem];
	[menuItem release];
	
	if(withDefault){
		[encryptionMenu addItem:[NSMenuItem separatorItem]];
		
		NSMenuItem *menuItem = [[NSMenuItem alloc] initWithTitle:AILocalizedString(@"Default",nil)
														  target:target
														  action:@selector(selectedEncryptionPreference:)
												   keyEquivalent:@""];
		
		[menuItem setTag:EncryptedChat_Default];
		[encryptionMenu addItem:menuItem];
		[menuItem release];
	}
	
	return [encryptionMenu autorelease];
}

- (NSImage *)imageForEventID:(NSString *)eventID
{
	static NSImage	*eventImage = nil;
	if(!eventImage) eventImage = [[NSImage imageNamed:@"message" forClass:[self class]] retain];
	return eventImage;
}

#pragma mark Ignore
- (void)toggleIgnoreOfContact:(id)sender
{
	AIListObject	*listObject = [[adium menuController] currentContextMenuObject];
	AIChat			*chat = [[adium menuController] currentContextMenuChat];
	
	if ([listObject isKindOfClass:[AIListContact class]]) {
		BOOL			isIgnored = [chat isListContactIgnored:(AIListContact *)listObject];
		[chat setListContact:(AIListContact *)listObject isIgnored:!isIgnored];
	}
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	if (menuItem == menuItem_ignore) {
		AIListObject	*listObject = [[adium menuController] currentContextMenuObject];
		AIChat			*chat = [[adium menuController] currentContextMenuChat];
		
		if ([listObject isKindOfClass:[AIListContact class]]) {
			if ([chat isListContactIgnored:(AIListContact *)listObject]) {
				[menuItem setTitle:AILocalizedString(@"Un-ignore","Un-ignore means begin receiving messages from this contact again in a chat")];
				
			} else {
				[menuItem setTitle:AILocalizedString(@"Ignore","Ignore means no longer receive messages from this contact in a chat")];
			}
		}
	}
	
	return YES;
}

#pragma mark Chat contact addition and removal

/*!
 * @brief A chat added a listContact to its participatants list
 *
 * @param chat The chat
 * @param inContact The contact
 * @param notify If YES, trigger the contact joined event if this is a group chat.  Ignored if this is not a group chat.
 */
- (void)chat:(AIChat *)chat addedListContact:(AIListContact *)inContact notify:(BOOL)notify
{
	if (notify && [chat name]) {
		/* Prevent triggering of the event when we are informed that the chat's own account entered the chat
		* If the UID of a contact in a chat differs from a normal UID, such as is the case with Jabber where a chat
		* contact has the form "roomname@conferenceserver/handle" this will fail, but it's better than nothing.
		*/
		if (![[[inContact account] UID] isEqualToString:[inContact UID]]) {
			[adiumChatEvents chat:chat addedListContact:inContact];
			
			[[adium contentController] displayStatusMessage:[NSString stringWithFormat:AILocalizedString(@"%@ joined the chat",nil),[inContact displayName]]
													 ofType:@"contact_joined"
													 inChat:chat];
		}
	}
	
	//Always notify Adium that the list changed so it can be updated, caches can be modified, etc.
	[[adium notificationCenter] postNotificationName:Chat_ParticipatingListObjectsChanged
											  object:chat];
}

/*!
 * @brief A chat removed a listContact from its participants list
 *
 * @param chat The chat
 * @param inContact The contact
 */
- (void)chat:(AIChat *)chat removedListContact:(AIListContact *)inContact
{
	if ([chat name]) {
		[adiumChatEvents chat:chat removedListContact:inContact];
		
		[[adium contentController] displayStatusMessage:[NSString stringWithFormat:AILocalizedString(@"%@ left the chat.",nil),[inContact displayName]]
												 ofType:@"contact_left"
												 inChat:chat];		
	}
	
	[[adium notificationCenter] postNotificationName:Chat_ParticipatingListObjectsChanged
											  object:chat];
}



@end
